/* Scrutari - Copyright (c) 2009-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets;

import fr.exemole.sctserver.api.EngineContext;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.api.context.EngineGroup;
import fr.exemole.sctserver.api.storage.EngineStorage;
import fr.exemole.sctserver.directory.DirectoryEngineStorage;
import fr.exemole.sctserver.impl.SctEngineImpl;
import fr.exemole.sctserver.tools.ErrorUtils;
import fr.exemole.sctserver.tools.SctMessageLocalisationFactory;
import fr.exemole.sctserver.tools.context.EngineGroupDirectory;
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;
import java.util.TreeMap;
import java.util.TreeSet;
import javax.servlet.ServletContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangPreference;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.localisation.MessageLocalisationFactory;
import net.mapeadores.util.logging.MessageLog;
import net.mapeadores.util.logging.MessageLogBuilder;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.XMLUtils;
import net.scrutari.db.api.ScrutariDBName;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariWebapp implements EngineContext {

    public final static String SCRUTARIWEBAPP_KEY = "fr.exemole.ScrutariWebapp";
    private final static Lang DEFAULT_LANG_UI = Lang.build("fr");
    private final String scrutariConfFileString;
    private final Timer timer;
    private final MessageLocalisationFactory messageLocalisationFactory;
    private final Set<String> engineNameSet = new TreeSet<String>();
    private final Map<String, SctEngineImpl> engineMap = new TreeMap<String, SctEngineImpl>();
    private final Map<String, EngineGroup> engineGroupMap = new TreeMap<String, EngineGroup>();
    private String uuidString = "";
    private String urlString = "";
    private String confDirString = "";
    private String varDirString = "";
    private File confDir = null;
    private File varDir = null;
    private MessageLog messageLog;
    private Lang webappDefaultLangUi = DEFAULT_LANG_UI;

    ScrutariWebapp(String scrutariConfFileString) {
        this.scrutariConfFileString = scrutariConfFileString;
        this.messageLocalisationFactory = SctMessageLocalisationFactory.build();
        reload();
        migrate();
        this.timer = new Timer(true);
        timer.scheduleAtFixedRate(new UpdateTask(), getFirstTime(), getPeriod());
    }

    public MessageLog getInitMessageLog() {
        return messageLog;
    }

    public List<String> getEngineNameList() {
        ArrayList<String> list = new ArrayList<String>();
        list.addAll(engineNameSet);
        return list;
    }

    @Override
    public SctEngine getEngine(String engineName) {
        SctEngineImpl engine = engineMap.get(engineName);
        if (engine == null) {
            engine = createEngine(engineName);
        }
        return engine;
    }

    @Override
    public Lang getWebappDefaultLangUi() {
        return webappDefaultLangUi;
    }

    @Override
    public String getWebappUUID() {
        return uuidString;
    }

    @Override
    public String getWebappUrl() {
        return urlString;
    }

    @Override
    public EngineGroup getEngineGroup(String groupName) {
        return engineGroupMap.get(groupName);
    }

    @Override
    public MessageLocalisation getMessageLocalisation(LangPreference langPreference) {
        return messageLocalisationFactory.newInstance(langPreference);
    }

    private SctEngineImpl createEngine(String engineName) {
        if (!engineNameSet.contains(engineName)) {
            return null;
        }
        engineName = engineName.intern();
        SctEngineImpl engine;
        synchronized (engineName) {
            engine = engineMap.get(engineName);
            if (engine == null) {
                engine = buildEngine(engineName);
                synchronized (engineMap) {
                    engineMap.put(engineName, engine);
                }
            }
        }
        return engine;
    }

    public static ScrutariWebapp getScrutariWebapp(ServletContext servletContext) {
        return (ScrutariWebapp) servletContext.getAttribute(SCRUTARIWEBAPP_KEY);
    }

    public void contextDestroyed() {
        timer.cancel();
    }

    public final synchronized boolean reload() {
        MessageLogBuilder messageLogBuilder = new MessageLogBuilder();
        boolean done = reload(messageLogBuilder);
        this.messageLog = messageLogBuilder.toMessageLog();
        return done;
    }

    private boolean reload(MessageLogBuilder messageLogBuilder) {
        messageLogBuilder.setCurrentURI("CATALINA:context.xml");
        if (scrutariConfFileString == null) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.initparameter");
            return false;
        }
        messageLogBuilder.setCurrentURI(scrutariConfFileString);
        File scrutariConfFile = new File(scrutariConfFileString);
        if (!scrutariConfFile.exists()) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.conffile_missing", scrutariConfFileString);
            return false;
        }
        if (scrutariConfFile.isDirectory()) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.conffile_dir", scrutariConfFileString);
            return false;
        }
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(scrutariConfFile);
            return readConf(doc.getDocumentElement(), messageLogBuilder);
        } catch (ParserConfigurationException pce) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.parseconfig", pce.getMessage());
            return false;
        } catch (IOException ioe) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.conffile_io", scrutariConfFileString, ioe.getMessage());
            return false;
        } catch (SAXException sax) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.conffile_xml", scrutariConfFileString, sax.getMessage());
            return false;
        }
    }


    private boolean readConf(Element element_xml, MessageLogBuilder messageLogBuilder) {
        String nvUuidString = null;
        String nvConfDirString = null;
        String nvVarDirString = null;
        String nvUrlString = null;
        NodeList nodeList = element_xml.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node nd = nodeList.item(i);
            if (nd instanceof Element) {
                Element el = (Element) nd;
                String tagName = el.getTagName();
                String data = XMLUtils.getData(el);
                if (tagName.equals("conf-dir")) {
                    nvConfDirString = data;
                } else if (tagName.equals("var-dir")) {
                    nvVarDirString = data;
                } else if (tagName.equals("uuid")) {
                    nvUuidString = data;
                } else if (tagName.equals("url")) {
                    nvUrlString = data;
                    if (nvUrlString.length() > 0) {
                        if (!nvUrlString.endsWith("/")) {
                            nvUrlString = nvUrlString + "/";
                        }
                    }
                } else if (tagName.equals("default-lang-ui")) {
                    if (data.length() > 0) {
                        try {
                            webappDefaultLangUi = Lang.parse(data);
                        } catch (ParseException pe) {
                            ErrorUtils.severe(messageLogBuilder, "_ error.wrong.xml.tagvalue", "/scrutari-conf/default-lang-ui", data);
                            webappDefaultLangUi = DEFAULT_LANG_UI;
                        }
                    } else {
                        webappDefaultLangUi = DEFAULT_LANG_UI;
                    }
                }
            }
        }
        boolean globalChange = false;
        boolean critical = false;
        if ((nvUuidString == null) || (nvUuidString.length() == 0)) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.xml_missingtag", "/scrutari-conf/uuid");
            critical = true;
        }
        if ((nvConfDirString == null) || (nvConfDirString.length() == 0)) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.xml_missingtag", "/scrutari-conf/conf-dir");
            critical = true;
        }
        if ((nvVarDirString == null) || (nvVarDirString.length() == 0)) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.xml_missingtag", "/scrutari-conf/var-dir");
            critical = true;
        }
        if ((nvUrlString == null) || (nvUrlString.length() == 0)) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.xml_missingtag", "/scrutari-conf/url");
            critical = true;
        }
        if (critical) {
            return false;
        }
        if (!nvUuidString.equals(uuidString)) {
            uuidString = nvUuidString;
            globalChange = true;
        }
        if (!nvUrlString.equals(urlString)) {
            urlString = nvUrlString;
            globalChange = true;
        }
        if (!nvConfDirString.equals(confDirString)) {
            File dir = testDir(nvConfDirString, "<conf-dir>", messageLogBuilder);
            if (dir != null) {
                confDirString = nvConfDirString;
                confDir = dir;
                globalChange = true;
            } else {
                return false;
            }
        }
        if (!nvVarDirString.equals(varDirString)) {
            File dir = testDir(nvVarDirString, "<var-dir>", messageLogBuilder);
            if (dir != null) {
                varDirString = nvVarDirString;
                varDir = dir;
                globalChange = true;
            } else {
                return false;
            }
        }
        boolean groupsHere = false;
        Set<String> hereSet = new HashSet<String>();
        for (File dir : confDir.listFiles()) {
            if (!dir.isDirectory()) {
                continue;
            }
            String name = dir.getName();
            if (name.equals("_groups")) {
                Map<String, EngineGroup> newEngineGroupMap = EngineGroupDirectory.read(dir, messageLogBuilder);
                engineGroupMap.clear();
                engineGroupMap.putAll(newEngineGroupMap);
                groupsHere = true;
            }
            if ((!name.startsWith("_")) && (StringUtils.isTechnicalName(name, true))) {
                hereSet.add(name);
            }
        }
        if (!groupsHere) {
            engineGroupMap.clear();
        }
        engineNameSet.clear();
        engineNameSet.addAll(hereSet);
        synchronized (engineMap) {
            if (globalChange) {
                engineMap.clear();
            } else {
                Map<String, SctEngineImpl> newMap = new HashMap<String, SctEngineImpl>();
                for (String engineName : hereSet) {
                    SctEngineImpl engine = engineMap.get(engineName);
                    if (engine != null) {
                        newMap.put(engineName, engine);
                    }
                }
                engineMap.clear();
                engineMap.putAll(newMap);
            }
        }
        return true;
    }

    private File testDir(String path, String elTag, MessageLogBuilder messageLogBuilder) {
        File dir = new File(path);
        if (!dir.exists()) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.xml_path_notexist", elTag, path);
            return null;
        }
        if (!dir.isDirectory()) {
            ErrorUtils.critical(messageLogBuilder, "_ error.config.xml_path_notdir", elTag, path);
            return null;
        }
        return dir;
    }

    private Date getFirstTime() {
        Calendar calendar = Calendar.getInstance();
        int day = calendar.get(Calendar.DAY_OF_MONTH);
        calendar.set(Calendar.DAY_OF_MONTH, day + 1);
        calendar.set(Calendar.HOUR_OF_DAY, 1);
        calendar.set(Calendar.MINUTE, 11);
        return calendar.getTime();
    }

    private long getPeriod() {
        return 24 * 60 * 60 * 1000;
    }

    private SctEngineImpl buildEngine(String engineName) {
        File engineConfDir = new File(confDir, engineName);
        File engineVarDir = new File(varDir, engineName);
        if (!engineVarDir.exists()) {
            engineVarDir.mkdirs();
        }
        EngineStorage engineStorage = new DirectoryEngineStorage(engineConfDir, engineVarDir);
        return new SctEngineImpl(engineName, engineStorage, this);
    }

    private void migrate() {
        //Révision 3109 (2017-01-17) : simplification du nom d'une session
        if (varDir.exists()) {
            for (File child : varDir.listFiles()) {
                migrateR3109(child);
            }
        }
    }

    private void migrateR3109(File file) {
        if (file.isDirectory()) {
            for (File child : file.listFiles()) {
                migrateR3109(child);
            }
        }
        String name = file.getName();
        String extension = null;
        int idx = name.lastIndexOf('.');
        if (idx > 1) {
            extension = name.substring(idx);
            name = name.substring(0, idx);
        }
        if (name.length() > 14) {
            try {
                ScrutariDBName scrutariDBName = ScrutariDBName.parse(name);
                String newName = scrutariDBName.getName();
                if (extension != null) {
                    newName = newName + extension;
                }
                file.renameTo(new File(file.getParentFile(), newName));
            } catch (ParseException pe) {

            }
        }
    }


    private class UpdateTask extends TimerTask {

        private UpdateTask() {
        }

        @Override
        public void run() {
            SctEngineImpl[] engineArray;
            synchronized (engineMap) {
                engineArray = engineMap.values().toArray(new SctEngineImpl[engineMap.size()]);
            }
            for (SctEngineImpl engine : engineArray) {
                engine.reload(SctEngine.CRON_TYPE);
            }
        }

    }

}
