/* Scrutari - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages;

import fr.exemole.sctserver.api.ConfState;
import fr.exemole.sctserver.api.EngineMetadata;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.api.conf.ConfConstants;
import fr.exemole.sctserver.api.log.SctLogConstants;
import fr.exemole.sctserver.servlets.admin.ConfUpdateCommand;
import fr.exemole.sctserver.servlets.admin.RestartCommand;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.AttributesPrint;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.CategoriesPrint;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.MetadataPrint;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.SourcesPrint;
import fr.exemole.sctserver.servlets.admin.htmlpages.utils.UriCodesPrint;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.attr.AttributeDefList;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.html.CommandBox;
import net.mapeadores.util.html.CommandBoxParametersBuilder;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HTMLAttributes;
import net.mapeadores.util.html.HtmlConstants;
import net.mapeadores.util.html.HtmlPrinterUtils;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.logging.CategorizedMessageList;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MessageLog;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.PertinencePonderation;


/**
 *
 * @author Vincent Calame
 */
public class IndexPage extends AdminHtmlPage {

    private final static HTMLAttributes ERROR_ATTR = HA.classes("scrutari-admin-ErrorMessage");
    private final ConfState confState;

    public IndexPage(SctEngine engine) {
        init(engine);
        confState = engine.getConfState();
        CommandBoxParametersBuilder commandBoxParametersBuilder = new CommandBoxParametersBuilder();
        commandBoxParametersBuilder.setCommandKeyDivAttributes(HA.classes("scrutari-admin-CommandKey"));
        commandBoxParametersBuilder.setFormAttributes(HA.action(".").method(HtmlConstants.GET_METHOD));
        commandBoxParametersBuilder.setSubmitDivAttributes(HA.classes("scrutari-admin-Submit"));
        setCommandBoxParameters(commandBoxParametersBuilder.toCommandBoxParameters());
    }

    @Override
    public void handleResponse(HttpServletResponse response) throws IOException {
        initResponse(response);
        start();
        printTools();
        printMessage();
        String dataLog = engine.getEngineStorage().getLogStorage().getLog(SctLogConstants.UPDATE_LOG, "data.xml");
        if (dataLog != null) {
            MessageLog messageLog = LogUtils.readMessageLog(dataLog);
            int count = messageLog.getLogGroupCount();
            if (count > 0) {
                startP();
                startSpan(ERROR_ATTR);
                localize("_ info.admin.updateprocesslog");
                endSpan();
                endP();
                startUl();
                for (int i = 0; i < count; i++) {
                    MessageLog.LogGroup logGroup = messageLog.getLogGroup(i);
                    startLi();
                    startP();
                    startCode(null);
                    escape('[');
                    escape(logGroup.getURI());
                    escape(']');
                    endCode();
                    endP();
                    CategorizedMessageList messageList = logGroup.getList();
                    int messageCount = messageList.getMessageCount();
                    int maxMessageCount = messageCount;
                    boolean withMax = false;
                    if (messageCount > 10) {
                        maxMessageCount = 5;
                        withMax = true;
                    }
                    startUl();
                    for (int j = 0; j < maxMessageCount; j++) {
                        Message message = messageList.getMessage(j);
                        String category = messageList.getCategory(j);
                        startLi();
                        startP();
                        if (category.length() > 0) {
                            startCode();
                            escape('[');
                            escape(category);
                            escape(']');
                            endCode();
                            space();
                        }
                        localize(message);
                        endP();
                        endLi();
                    }
                    if (withMax) {
                        startLi();
                        startP();
                        localize(LocalisationUtils.toMessage("_ info.admin.moreerrors", (messageCount - 5)));
                        endP();
                        endLi();
                    }
                    endUl();
                    endLi();
                }
                endUl();
            }
        }

        printRestartForm();

        printConfState();

        end();
    }

    private void printConfState() {
        startH2(null);
        localize("_ title.sctconf.files");
        endH2();
        short state = confState.getGlobalState();
        boolean printConfDetails = false;
        switch (state) {
            case ConfState.GLOBAL_CONFIO_ERROR:
                startP();
                startSpan(ERROR_ATTR);
                localize("_ error.global.conf_io");
                escape(" : ");
                escape(confState.getConfIOMessage());
                endSpan();
                endP();
                break;
            case ConfState.GLOBAL_UNDEFINED:
                startP();
                startSpan(ERROR_ATTR);
                localize("_ error.global.conf_null");
                escape(" : ");
                escape(confState.getConfIOMessage());
                endSpan();
                endP();
                break;
            case ConfState.GLOBAL_OK:
                startUl();
                printConfFileLi(ConfConstants.SOURCES_NAME);
                printConfFileLi(ConfConstants.METADATA_NAME);
                printConfFileLi(ConfConstants.ATTRIBUTES_NAME);
                printConfFileLi(ConfConstants.CATEGORIES_NAME);
                printConfFileLi(ConfConstants.URI_CODES_NAME);
                printConfFileLi(ConfConstants.FEED_NAME);
                printConfFileLi(ConfConstants.OPTIONS_NAME);
                printConfFileLi(ConfConstants.VALIDATION_NAME);
                endUl();
                printConfDetails = true;
                break;
        }
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction(".");
        commandBox.setCommandFamily("SCT");
        commandBox.setCommandString(ConfUpdateCommand.COMMANDNAME);
        commandBox.setCommandKey(ConfUpdateCommand.COMMANDKEY);
        startCommandBox(commandBox);
        endCommandBox("_ submit.admin.confupdate");
        if (printConfDetails) {
            startH2(null);
            localize("_ title.sctconf.details");
            endH2();
            printSourcesDetails();
            printMetadataDetails();
            printAttributesDetails();
            printCategoriesDetails();
            printUriCodesDetails();
            printFeedDetails();
            printOptionsDetails();
            printValidationDetails();
        }
    }

    private void printConfFileLi(String name) {
        startLi();
        printFileInfo(name, true);
        endLi();
    }

    private void printSourcesDetails() {
        startH3(HA.id(ConfConstants.SOURCES_NAME));
        localize("_ title.sctconf.sources");
        escape(" / ");
        printFileInfo(ConfConstants.SOURCES_NAME, false);
        endH3();
        printConfLog(ConfConstants.SOURCES_NAME);
        SourcesPrint.printScrutariSourceList(this);
    }

    private void printMetadataDetails() {
        startH3(HA.id(ConfConstants.METADATA_NAME));
        localize("_ title.sctconf.metadata");
        escape(" / ");
        printFileInfo(ConfConstants.METADATA_NAME, false);
        endH3();
        printConfLog(ConfConstants.METADATA_NAME);
        EngineMetadata engineMetadata = engine.getEngineMetadata();
        MetadataPrint.printLabelHolder(this, "title", engineMetadata.getTitle());
        MetadataPrint.printLabelHolder(this, "description", engineMetadata.getDescription());
        MetadataPrint.printUrl(this, "icon", engineMetadata.getIcon());
        MetadataPrint.printUrl(this, "website", engineMetadata.getWebsite());
        MetadataPrint.printAttributeHolder(this, engineMetadata);
    }

    private void printAttributesDetails() {
        startH3(HA.id(ConfConstants.ATTRIBUTES_NAME));
        localize("_ title.sctconf.attributes");
        escape(" / ");
        printFileInfo(ConfConstants.ATTRIBUTES_NAME, false);
        endH3();
        printConfLog(ConfConstants.ATTRIBUTES_NAME);
        AttributeKeyCoder attributeKeyCoder = engine.getScrutariSession().getScrutariDB().getAttributeKeyCoder();
        if (attributeKeyCoder.getAttributeDefCount() == 0) {
            startP();
            startSpan(HA.classes("scrutari-admin-Info"));
            localize("_ info.sctconf.none_attributes");
            endSpan();
            endP();
            return;
        }
        printAttributesList(attributeKeyCoder.getAttributeDefList(AttributeKeyCoder.PRIMARY_GROUP), "_ title.sctconf.attributes_primary");
        printAttributesList(attributeKeyCoder.getAttributeDefList(AttributeKeyCoder.SECONDARY_GROUP), "_ title.sctconf.attributes_secondary");
        printAttributesList(attributeKeyCoder.getAttributeDefList(AttributeKeyCoder.TECHNICAL_GROUP), "_ title.sctconf.attributes_technical");
    }

    private void printAttributesList(AttributeDefList attributeDefList, String titleMessageKey) {
        if (attributeDefList.getAttributeDefCount() == 0) {
            return;
        }
        startH4(null);
        localize(titleMessageKey);
        endH4();
        AttributesPrint.printAttributesList(this, attributeDefList);
    }


    private void printCategoriesDetails() {
        startH3(HA.id(ConfConstants.CATEGORIES_NAME));
        localize("_ title.sctconf.categories");
        escape(" / ");
        printFileInfo(ConfConstants.CATEGORIES_NAME, false);
        endH3();
        printConfLog(ConfConstants.CATEGORIES_NAME);
        CategoriesPrint.printCategories(this);
    }

    private void printUriCodesDetails() {
        startH3(HA.id(ConfConstants.URI_CODES_NAME));
        localize("_ title.sctconf.uricodes");
        escape(" / ");
        printFileInfo(ConfConstants.URI_CODES_NAME, false);
        endH3();
        printConfLog(ConfConstants.URI_CODES_NAME);
        UriCodesPrint.printUriCodes(this);
    }

    private void printFeedDetails() {
        startH3(HA.id(ConfConstants.FEED_NAME));
        localize("_ title.sctconf.feed");
        escape(" / ");
        printFileInfo(ConfConstants.FEED_NAME, false);
        endH3();
        printConfLog(ConfConstants.FEED_NAME);
    }

    private void printOptionsDetails() {
        startH3(HA.id(ConfConstants.OPTIONS_NAME));
        localize("_ title.sctconf.options");
        escape(" / ");
        printFileInfo(ConfConstants.OPTIONS_NAME, false);
        endH3();
        printConfLog(ConfConstants.OPTIONS_NAME);
        startH4(null);
        localize("_ title.sctconf.options_ponderation");
        endH4();
        GlobalSearchOptions globalSearchOptions = engine.getScrutariSession().getGlobalSearchOptions();
        PertinencePonderation pertinencePonderation = globalSearchOptions.getPertinencePonderation();
        startTable();
        printPonderatioRow("_ label.sctconf.ponderation_occurrence", pertinencePonderation.getOccurrencePonderation());
        printPonderatioRow("_ label.sctconf.ponderation_date", pertinencePonderation.getDatePonderation());
        printPonderatioRow("_ label.sctconf.ponderation_origin", pertinencePonderation.getOriginPonderation());
        endTable();
    }

    private void printValidationDetails() {
        startH3(HA.id(ConfConstants.VALIDATION_NAME));
        localize("_ title.sctconf.validation");
        escape(" / ");
        printFileInfo(ConfConstants.VALIDATION_NAME, false);
        endH3();
        printConfLog(ConfConstants.VALIDATION_NAME);
    }

    private void printPonderatioRow(String key, float f) {
        startRow();
        startCell();
        HtmlPrinterUtils.localizeWithColon(this, key);
        endCell();
        startCell();
        escape(String.valueOf(Math.round(f * 100)));
        escape('%');
        endCell();
        endRow();
    }

    private void printRestartForm() throws IOException {
        CommandBox commandBox = new CommandBox();
        commandBox.setFormAction(".");
        commandBox.setCommandFamily("SCT");
        commandBox.setCommandString(RestartCommand.COMMANDNAME);
        commandBox.setCommandKey(RestartCommand.COMMANDKEY);
        startCommandBox(commandBox);
        startP();
        startLabel(null);
        appendInput(HA.type(HtmlConstants.CHECKBOX_TYPE).name(RestartCommand.FORCE_PARAMNAME).value("1"));
        localize("_ label.admin.forcerestart");
        endLabel();
        endP();
        endCommandBox("_ submit.admin.restart");
    }

    private void printConfLog(String name) {
        short fileState = confState.getFileState(name);
        switch (fileState) {
            case ConfConstants.FILE_STATE_UNKNOWN:
            case ConfConstants.FILE_MISSING:
            case ConfConstants.FILE_OK:
                return;
        }
        String logContent = engine.getEngineStorage().getLogStorage().getLog(SctLogConstants.CONF_LOG, name + ".xml");
        if (logContent == null) {
            return;
        }
        CategorizedMessageList processMessageList = LogUtils.readCategorizedMessageList(logContent);
        int count = processMessageList.getMessageCount();
        if (count > 0) {
            startH4(null);
            localize("_ title.admin.log");
            endH4();
            startUl();
            for (int i = 0; i < count; i++) {
                Message message = processMessageList.getMessage(i);
                String category = processMessageList.getCategory(i);
                startLi();
                startP();
                if (category.length() > 0) {
                    startCode();
                    escape('[');
                    escape(category);
                    escape(']');
                    endCode();
                    space();
                }
                localize(message);
                endP();
                endLi();
            }
            endUl();
        }
    }

    private void printFileInfo(String name, boolean withLink) {
        short fileState = confState.getFileState(name);
        if (withLink) {
            startA(HA.href("#" + name));
        }
        escape(name);
        escape(".xml");
        if (withLink) {
            endA();
        }
        space();
        escape('(');
        startSpan(HA.classes(getSpanClass(fileState)));
        localize(getFileStateMessageKey(fileState));
        endSpan();
        escape(')');
    }

    private static String getSpanClass(short fileState) {
        switch (fileState) {
            case ConfConstants.FILE_STATE_UNKNOWN:
                return "scrutari-state-Neutral";
            case ConfConstants.FILE_MISSING:
                return "scrutari-state-Neutral";
            case ConfConstants.FILE_XML_MALFORMED:
                return "scrutari-state-Error";
            case ConfConstants.FILE_XML_ERROR:
                return "scrutari-state-Error";
            case ConfConstants.FILE_XML_WARNING:
                return "scrutari-state-Warning";
            case ConfConstants.FILE_OK:
                return "scrutari-state-Ok";
            default:
                throw new SwitchException("Unknown getFileState : " + fileState);
        }
    }

    private static String getFileStateMessageKey(short fileState) {
        switch (fileState) {
            case ConfConstants.FILE_STATE_UNKNOWN:
                return "_ state.file.unknownstate";
            case ConfConstants.FILE_MISSING:
                return "_ state.file.missing";
            case ConfConstants.FILE_XML_MALFORMED:
                return "_ state.file.xml_malformed";
            case ConfConstants.FILE_XML_ERROR:
                return "_ state.file.xml_error";
            case ConfConstants.FILE_XML_WARNING:
                return "_ state.file.xml_warning";
            case ConfConstants.FILE_OK:
                return "_ state.file.ok";
            default:
                throw new SwitchException("Unknown getFileState : " + fileState);
        }
    }

}
