/* ScrutariLib_DB - Copyright (c) 2005-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.primitives.DuoKey;
import net.mapeadores.util.primitives.FuzzyDate;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.IndexationData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.ScrutariDataURIChecker;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.DataAccessProvider;
import net.scrutari.db.api.caches.CacheIterator;
import net.scrutari.db.api.caches.DataCacheReader;
import net.scrutari.db.api.caches.DataCacheReaderFactory;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.IndexationCodeList;
import net.scrutari.db.tools.codes.CodesUtils;


/**
 * Dans toutes les fonctions d'ajout, on suppose qu'il n'y a pas de doublons.
 *
 * @author Vincent Calame
 */
public class DataAccessProviderBuilder {

    private DataAccessProviderBuilder() {
    }

    public static DataAccessProvider build(DataCacheReaderFactory dataCacheReaderFactory, ScrutariDataURIChecker scrutariDataURIChecker) {
        Map<ScrutariDataURI, Integer> codeByScrutariDataURIMap = new HashMap<ScrutariDataURI, Integer>();
        Map<Integer, Object> dataMap = new HashMap<Integer, Object>();
        Map<ScrutariDataURI, ScrutariDataURI> internMap = new HashMap<ScrutariDataURI, ScrutariDataURI>();
        List<BaseData> baseDataList = new ArrayList<BaseData>();
        List<CorpusData> corpusDataList = new ArrayList<CorpusData>();
        List<ThesaurusData> thesaurusDataList = new ArrayList<ThesaurusData>();
        CacheIterator<BaseData> baseDataIterator = dataCacheReaderFactory.newBaseDataIterator();
        while (baseDataIterator.hasNext()) {
            BaseData baseData = baseDataIterator.next();
            int codebase = baseData.getCodeBase();
            BaseURI baseURI = baseData.getBaseURI();
            dataMap.put(codebase, baseData);
            baseDataList.add(baseData);
            internMap.put(baseURI, baseURI);
            codeByScrutariDataURIMap.put(baseURI, codebase);
        }
        baseDataIterator.close();
        CacheIterator<CorpusData> corpusDataIterator = dataCacheReaderFactory.newCorpusDataIterator();
        while (corpusDataIterator.hasNext()) {
            CorpusData corpusData = corpusDataIterator.next();
            int codecorpus = corpusData.getCodeCorpus();
            CorpusURI corpusURI = corpusData.getCorpusURI();
            corpusURI = corpusURI.derive((BaseURI) internMap.get(corpusURI.getBaseURI()));
            dataMap.put(codecorpus, corpusData);
            corpusDataList.add(corpusData);
            internMap.put(corpusURI, corpusURI);
            codeByScrutariDataURIMap.put(corpusURI, codecorpus);
        }
        corpusDataIterator.close();
        CacheIterator<ThesaurusData> thesaurusDataIterator = dataCacheReaderFactory.newThesaurusDataIterator();
        while (thesaurusDataIterator.hasNext()) {
            ThesaurusData thesaurusData = thesaurusDataIterator.next();
            int codethesaurus = thesaurusData.getCodeThesaurus();
            ThesaurusURI thesaurusURI = thesaurusData.getThesaurusURI();
            thesaurusURI = thesaurusURI.derive((BaseURI) internMap.get(thesaurusURI.getBaseURI()));
            dataMap.put(codethesaurus, thesaurusData);
            thesaurusDataList.add(thesaurusData);
            internMap.put(thesaurusURI, thesaurusURI);
            codeByScrutariDataURIMap.put(thesaurusURI, codethesaurus);
        }
        thesaurusDataIterator.close();
        CacheIterator<FicheData> ficheDataIterator = dataCacheReaderFactory.newFicheDataIterator();
        while (ficheDataIterator.hasNext()) {
            FicheData ficheData = ficheDataIterator.next();
            int codefiche = ficheData.getCodeFiche();
            FicheURI ficheURI = ficheData.getFicheURI();
            ficheURI = ficheURI.derive((CorpusURI) internMap.get(ficheURI.getCorpusURI()));
            codeByScrutariDataURIMap.put(ficheURI, codefiche);
        }
        ficheDataIterator.close();
        CacheIterator<MotcleData> motcleDataIterator = dataCacheReaderFactory.newMotcleDataIterator();
        while (motcleDataIterator.hasNext()) {
            MotcleData motcleData = motcleDataIterator.next();
            int codemotcle = motcleData.getCodeMotcle();
            MotcleURI motcleURI = motcleData.getMotcleURI();
            motcleURI = motcleURI.derive((ThesaurusURI) internMap.get(motcleURI.getThesaurusURI()));
            codeByScrutariDataURIMap.put(motcleURI, codemotcle);
        }
        motcleDataIterator.close();
        Map<DuoKey, IndexationCodeList> indexationMap = initIndexation(dataCacheReaderFactory);
        BaseData[] baseDataArray = baseDataList.toArray(new BaseData[baseDataList.size()]);
        CorpusData[] corpusDataArray = corpusDataList.toArray(new CorpusData[corpusDataList.size()]);
        ThesaurusData[] thesaurusDataArray = thesaurusDataList.toArray(new ThesaurusData[thesaurusDataList.size()]);
        return new InternalDataAccessProvider(dataCacheReaderFactory, scrutariDataURIChecker, dataMap, baseDataArray, corpusDataArray, thesaurusDataArray, codeByScrutariDataURIMap, indexationMap);
    }

    private static Map<DuoKey, IndexationCodeList> initIndexation(DataCacheReaderFactory dataCacheReaderFactory) {
        Map<DuoKey, Integer> poidsMaxMap = new HashMap<DuoKey, Integer>();
        CacheIterator<IndexationData> iterator = dataCacheReaderFactory.newIndexationDataIterator();
        while (iterator.hasNext()) {
            IndexationData indexationData = iterator.next();
            int poids = indexationData.getPoids();
            if (poids > 1) {
                DuoKey duoKey = new DuoKey(indexationData.getCodeCorpus(), indexationData.getCodeThesaurus());
                Integer current = poidsMaxMap.get(duoKey);
                if ((current == null) || (current < poids)) {
                    poidsMaxMap.put(duoKey, poids);
                }
            }
        }
        iterator.close();
        Map<DuoKey, IndexationCodeListBuilder> builderMap = new HashMap<DuoKey, IndexationCodeListBuilder>();
        iterator = dataCacheReaderFactory.newIndexationDataIterator();
        while (iterator.hasNext()) {
            IndexationData indexationData = iterator.next();
            int codefiche = indexationData.getCodeFiche();
            int codemotcle = indexationData.getCodeMotcle();
            int codecorpus = indexationData.getCodeCorpus();
            int codethesaurus = indexationData.getCodeThesaurus();
            DuoKey duoKey = new DuoKey(codecorpus, codethesaurus);
            Integer poidsMaxItg = poidsMaxMap.get(duoKey);
            int poidsMax = 1;
            if (poidsMaxItg != null) {
                poidsMax = poidsMaxItg;
            }
            float ponderation = ((float) indexationData.getPoids()) / poidsMax;
            DuoKey ficheDuoKey = new DuoKey(codefiche, codethesaurus);
            IndexationCodeListBuilder ficheListBuilder = builderMap.get(ficheDuoKey);
            if (ficheListBuilder == null) {
                ficheListBuilder = new IndexationCodeListBuilder();
                builderMap.put(ficheDuoKey, ficheListBuilder);
            }
            ficheListBuilder.add(codemotcle, ponderation);
            DuoKey motcleDuoKey = new DuoKey(codemotcle, codecorpus);
            IndexationCodeListBuilder motcleListBuilder = builderMap.get(motcleDuoKey);
            if (motcleListBuilder == null) {
                motcleListBuilder = new IndexationCodeListBuilder();
                builderMap.put(motcleDuoKey, motcleListBuilder);
            }
            motcleListBuilder.add(codefiche, ponderation);
        }
        iterator.close();
        Map<DuoKey, IndexationCodeList> result = new HashMap<DuoKey, IndexationCodeList>();
        for (Map.Entry<DuoKey, IndexationCodeListBuilder> entry : builderMap.entrySet()) {
            result.put(entry.getKey(), entry.getValue().toIndexationCodeList());
        }
        return result;
    }


    private static class IndexationCodeListBuilder {

        private final List<Integer> elementList = new ArrayList<Integer>();
        private final List<Float> ponderationList = new ArrayList<Float>();

        private void add(int elementCode, float ponderation) {
            Integer itg = elementCode;
            if (!elementList.contains(itg)) {
                elementList.add(itg);
                ponderationList.add(ponderation);
            }
        }

        private IndexationCodeList toIndexationCodeList() {
            return CodesUtils.toIndexationCodeList(elementList, ponderationList);
        }

    }


    private static class InternalDataAccessProvider implements DataAccessProvider {

        private final DataCacheReaderFactory dataCacheReaderFactory;
        private final Map<Integer, Object> dataMap;
        private final BaseData[] baseDataArray;
        private final CorpusData[] corpusDataArray;
        private final ThesaurusData[] thesaurusDataArray;
        private final Map<ScrutariDataURI, Integer> codeByScrutariDataURIMap;
        private final Map<DuoKey, IndexationCodeList> indexationMap;
        private final Map<String, BaseData> baseDataBySourceNameMap = new HashMap<String, BaseData>();
        private final CodeList corpusCodeList;
        private final CodeList thesaurusCodeList;
        private final ScrutariDataURIChecker scrutariDataURIChecker;

        private InternalDataAccessProvider(DataCacheReaderFactory dataCacheReaderFactory, ScrutariDataURIChecker scrutariDataURIChecker, Map<Integer, Object> dataMap, BaseData[] baseDataArray, CorpusData[] corpusDataArray, ThesaurusData[] thesaurusDataArray, Map<ScrutariDataURI, Integer> codeByScrutariDataURIMap, Map<DuoKey, IndexationCodeList> indexationMap) {
            this.dataCacheReaderFactory = dataCacheReaderFactory;
            this.scrutariDataURIChecker = scrutariDataURIChecker;
            this.dataMap = dataMap;
            this.baseDataArray = baseDataArray;
            this.corpusDataArray = corpusDataArray;
            this.thesaurusDataArray = thesaurusDataArray;
            this.codeByScrutariDataURIMap = codeByScrutariDataURIMap;
            this.indexationMap = indexationMap;
            for (BaseData baseData : baseDataArray) {
                baseDataBySourceNameMap.put(baseData.getScrutariSourceName(), baseData);
            }
            int corpusCount = corpusDataArray.length;
            int[] codescorpus = new int[corpusCount];
            for (int i = 0; i < corpusCount; i++) {
                codescorpus[i] = corpusDataArray[i].getCodeCorpus();
            }
            this.corpusCodeList = CodesUtils.toCodeList(codescorpus);
            int thesaurusCount = thesaurusDataArray.length;
            int[] codesthesaurus = new int[thesaurusCount];
            for (int i = 0; i < thesaurusCount; i++) {
                codesthesaurus[i] = thesaurusDataArray[i].getCodeThesaurus();
            }
            this.thesaurusCodeList = CodesUtils.toCodeList(codesthesaurus);
        }

        @Override
        public DataAccess openDataAccess() {
            return new InternalDataAccessProvider.InternalDataAccess();
        }


        private class InternalDataAccess implements DataAccess {

            private DataCacheReader dataCacheReader;

            private InternalDataAccess() {
            }

            @Override
            public CodeList getCorpusCodeList() {
                return corpusCodeList;
            }

            @Override
            public CodeList getThesaurusCodeList() {
                return thesaurusCodeList;
            }

            @Override
            public int getBaseCount() {
                return baseDataArray.length;
            }

            @Override
            public int getCorpusCount() {
                return corpusCodeList.getCodeCount();
            }

            @Override
            public int getThesaurusCount() {
                return thesaurusCodeList.getCodeCount();
            }

            @Override
            public int getCode(ScrutariDataURI scrutariDataURI) {
                scrutariDataURI = scrutariDataURIChecker.checkURI(scrutariDataURI);
                Integer code = codeByScrutariDataURIMap.get(scrutariDataURI);
                if (code == null) {
                    return -1;
                }
                return code;
            }

            @Override
            public ScrutariDataURIChecker getScrutariDataURIChecker() {
                return scrutariDataURIChecker;
            }

            @Override
            public Object getScrutariData(ScrutariDataURI scrutariDataURI) {
                scrutariDataURI = scrutariDataURIChecker.checkURI(scrutariDataURI);
                Integer code = codeByScrutariDataURIMap.get(scrutariDataURI);
                if (code == null) {
                    return null;
                }
                short type = scrutariDataURI.getType();
                switch (type) {
                    case ScrutariDataURI.BASEURI_TYPE:
                    case ScrutariDataURI.CORPUSURI_TYPE:
                    case ScrutariDataURI.THESAURUSURI_TYPE:
                        return dataMap.get(code);
                    case ScrutariDataURI.FICHEURI_TYPE:
                        return getFicheDataByCode(code);
                    case ScrutariDataURI.MOTCLEURI_TYPE:
                        return getMotcleDataByCode(code);
                    default:
                        throw new SwitchException("Unknown type: " + type);
                }
            }

            @Override
            public BaseData getBaseDataByCode(int codebase) {
                Object obj = dataMap.get(codebase);
                if ((obj == null) || (!(obj instanceof BaseData))) {
                    return null;
                }
                return (BaseData) obj;
            }

            @Override
            public BaseData getBaseDataByIndex(int index) {
                return baseDataArray[index];
            }

            @Override
            public BaseData getBaseDataByScrutariSourceName(String scrutariSourceName) {
                return baseDataBySourceNameMap.get(scrutariSourceName);
            }

            @Override
            public CorpusData getCorpusDataByCode(int codecorpus) {
                Object obj = dataMap.get(codecorpus);
                if ((obj == null) || (!(obj instanceof CorpusData))) {
                    return null;
                }
                return (CorpusData) obj;
            }

            @Override
            public CorpusData getCorpusDataByIndex(int index) {
                return corpusDataArray[index];
            }

            @Override
            public ThesaurusData getThesaurusDataByCode(int codethesaurus) {
                Object obj = dataMap.get(codethesaurus);
                if ((obj == null) || (!(obj instanceof ThesaurusData))) {
                    return null;
                }
                return (ThesaurusData) obj;
            }

            @Override
            public ThesaurusData getThesaurusDataByIndex(int index) {
                return thesaurusDataArray[index];
            }

            @Override
            public MotcleData getMotcleDataByCode(int codemotcle) {
                if (dataCacheReader == null) {
                    dataCacheReader = dataCacheReaderFactory.newReader();
                }
                return dataCacheReader.getMotcleData(codemotcle);
            }

            @Override
            public FicheData getFicheDataByCode(int codefiche) {
                if (dataCacheReader == null) {
                    dataCacheReader = dataCacheReaderFactory.newReader();
                }
                return dataCacheReader.getFicheData(codefiche);
            }

            @Override
            public Lang getFicheLang(int codefiche) {
                if (dataCacheReader == null) {
                    dataCacheReader = dataCacheReaderFactory.newReader();
                }
                return dataCacheReader.getFicheLang(codefiche);
            }

            @Override
            public FuzzyDate getFicheDate(int codefiche) {
                if (dataCacheReader == null) {
                    dataCacheReader = dataCacheReaderFactory.newReader();
                }
                return dataCacheReader.getFicheDate(codefiche);
            }

            @Override
            public IndexationCodeList getIndexationCodeList(int elementCode, int subsetCode) {
                DuoKey duoKey = new DuoKey(elementCode, subsetCode);
                IndexationCodeList indexationCodeList = indexationMap.get(duoKey);
                if (indexationCodeList == null) {
                    return CodesUtils.EMPTY_INDEXATIONCODELIST;
                }
                return indexationCodeList;
            }

            @Override
            public void close() {
                if (dataCacheReader != null) {
                    dataCacheReader.close();
                }
            }

        }

    }

}
