/* ScrutariLib_DB - Copyright (c) 2013-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.codes;

import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.datauri.URIParseException;
import net.scrutari.datauri.URIParser;
import net.scrutari.db.api.CodeMessageKeys;
import net.scrutari.db.api.DataAccess;


/**
 *
 * @author Vincent Calame
 */
public final class CodeParser {

    private CodeParser() {
    }

    public static int toCode(DataAccess dataAccess, String s, MessageHandler messageHandler, short uriType) {
        int slashIndex = s.indexOf('/');
        if (slashIndex == -1) {
            return parseWithoutSlash(dataAccess, s, messageHandler, uriType);
        }
        if (slashIndex == 0) {
            return parseURI(dataAccess, ScrutariDataURI.typeToSchemeString(uriType) + ":" + s, messageHandler, uriType, s);
        }
        int colonIndex = s.indexOf(':');
        if ((colonIndex > 0) && (colonIndex < slashIndex)) {
            return parseURI(dataAccess, s, messageHandler, uriType, s);
        }
        if (uriType == ScrutariDataURI.BASEURI_TYPE) {
            addError(messageHandler, CodeMessageKeys.MALFORMED_URI, s);
            return -1;
        }
        String firstPart = s.substring(0, slashIndex);
        String secondPart = s.substring(slashIndex);
        ScrutariDataURI startScrutariDataURI;
        try {
            int firstCode = Integer.parseInt(firstPart);
            startScrutariDataURI = getParentScrutariDataURI(dataAccess, firstCode);
            if (startScrutariDataURI == null) {
                addError(messageHandler, CodeMessageKeys.UNKNOWN_URI, firstPart + " (" + s + ")");
                return -1;
            }
        } catch (NumberFormatException nfe) {
            BaseData baseData = dataAccess.getBaseDataByScrutariSourceName(firstPart);
            if (baseData != null) {
                startScrutariDataURI = baseData.getBaseURI();
            } else {
                addError(messageHandler, CodeMessageKeys.MALFORMED_NOT_INTEGER, firstPart + " (" + s + ")");
                return -1;
            }
        }
        String uriString = ScrutariDataURI.typeToSchemeString(uriType) + ':' + startScrutariDataURI.getPath() + secondPart;
        return parseURI(dataAccess, uriString, messageHandler, uriType, secondPart + " (" + s + ")");
    }

    private static int parseWithoutSlash(DataAccess dataAccess, String s, MessageHandler messageHandler, short uriType) {
        try {
            int code = Integer.parseInt(s);
            if (!testCode(dataAccess, code, uriType)) {
                addError(messageHandler, CodeMessageKeys.UNKNOWN_CODE, s);
                return -1;
            }
            return code;
        } catch (NumberFormatException nfe) {
            if (uriType == ScrutariDataURI.BASEURI_TYPE) {
                BaseData baseData = dataAccess.getBaseDataByScrutariSourceName(s);
                if (baseData != null) {
                    return baseData.getCodeBase();
                }
            }
            addError(messageHandler, CodeMessageKeys.MALFORMED_NOT_INTEGER, s);
            return -1;
        }
    }

    private static int parseURI(DataAccess dataAccess, String uriString, MessageHandler messageHandler, short uriType, String originalString) {
        try {
            ScrutariDataURI scrutariDataURI = URIParser.parse(uriString);
            if (scrutariDataURI.getType() != uriType) {
                addError(messageHandler, CodeMessageKeys.WRONG_URI_TYPE, originalString);
                return -1;
            }
            int code = dataAccess.getCode(scrutariDataURI);
            if (code == -1) {
                addError(messageHandler, CodeMessageKeys.UNKNOWN_URI, originalString);
            }
            return code;
        } catch (URIParseException uriParseException) {
            addError(messageHandler, CodeMessageKeys.MALFORMED_URI, uriString);
            return -1;
        }
    }

    private static void addError(MessageHandler messageHandler, String messageKey, Object messageValue) {
        messageHandler.addMessage(CodeMessageKeys.ERROR_CATEGORY, LocalisationUtils.toMessage(messageKey, messageValue));
    }

    private static boolean testCode(DataAccess dataAccess, int code, short uriType) {
        switch (uriType) {
            case ScrutariDataURI.BASEURI_TYPE:
                return (dataAccess.getBaseDataByCode(code) != null);
            case ScrutariDataURI.CORPUSURI_TYPE:
                return (dataAccess.getCorpusDataByCode(code) != null);
            case ScrutariDataURI.THESAURUSURI_TYPE:
                return (dataAccess.getThesaurusDataByCode(code) != null);
            case ScrutariDataURI.FICHEURI_TYPE:
                return (dataAccess.getFicheDataByCode(code) != null);
            case ScrutariDataURI.MOTCLEURI_TYPE:
                return (dataAccess.getMotcleDataByCode(code) != null);
            default:
                throw new SwitchException("uriType = " + uriType);
        }

    }

    private static ScrutariDataURI getParentScrutariDataURI(DataAccess dataAccess, int code) {
        BaseData baseData = dataAccess.getBaseDataByCode(code);
        if (baseData != null) {
            return baseData.getBaseURI();
        }
        CorpusData corpusData = dataAccess.getCorpusDataByCode(code);
        if (corpusData != null) {
            return corpusData.getCorpusURI();
        }
        ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(code);
        if (thesaurusData != null) {
            return thesaurusData.getThesaurusURI();
        }
        return null;
    }

}
