/* ScrutariLib_DB - Copyright (c) 2010-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.codes;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.ElementCodeList;
import net.scrutari.db.api.codes.IndexationCodeList;


/**
 *
 * @author Vincent Calame
 */
public class CodesUtils {

    public final static CodeList EMPTY_CODELIST = new EmptyCodeList();
    public final static ElementCodeList EMPTY_ELEMENTCODELIST = new EmptyElementCodeList();
    public final static IndexationCodeList EMPTY_INDEXATIONCODELIST = new EmptyIndexationCodeList();

    public static CodeList toCodeList(int[] intArray) {
        if (intArray == null) {
            return EMPTY_CODELIST;
        }
        if (intArray.length == 0) {
            return EMPTY_CODELIST;
        }
        return new ArrayCodeList(intArray);
    }

    public static CodeList toCodeList(Collection<Integer> codeCollection) {
        if (codeCollection == null) {
            return EMPTY_CODELIST;
        }
        int size = codeCollection.size();
        if (size == 0) {
            return EMPTY_CODELIST;
        }
        if (size == 1) {
            return new UniqueCodeList(codeCollection.iterator().next());
        }
        int[] result = new int[size];
        int p = 0;
        for (Integer itg : codeCollection) {
            result[p] = itg;
            p++;
        }
        return new ArrayCodeList(result);
    }

    public static ElementCodeList toElementCodeList(List<Integer> elementCodeList, List<Integer> subsetCodeList) {
        if (elementCodeList == null) {
            return EMPTY_ELEMENTCODELIST;
        }
        int size = elementCodeList.size();
        if (size == 0) {
            return EMPTY_ELEMENTCODELIST;
        }
        if (size == 1) {
            return new UniqueElementCodeList(elementCodeList.get(0), subsetCodeList.get(0));
        }
        int[] result = new int[size];
        int[] subsetResult = new int[size];
        for (int i = 0; i < size; i++) {
            result[i] = elementCodeList.get(i);
            subsetResult[i] = subsetCodeList.get(i);
        }
        return new ArrayElementCodeList(result, subsetResult);
    }

    public static ElementCodeList toElementCodeList(Map<Integer, Integer> elementCodeListMap) {
        if (elementCodeListMap == null) {
            return EMPTY_ELEMENTCODELIST;
        }
        int size = elementCodeListMap.size();
        if (size == 0) {
            return EMPTY_ELEMENTCODELIST;
        }
        int[] result = new int[size];
        int[] subsetResult = new int[size];
        int p = 0;
        for (Map.Entry<Integer, Integer> entry : elementCodeListMap.entrySet()) {
            result[p] = entry.getKey();
            subsetResult[p] = entry.getValue();
            p++;
        }
        return new ArrayElementCodeList(result, subsetResult);
    }

    public static IndexationCodeList toIndexationCodeList(List<Integer> elementCodeList, List<Float> ponderationList) {
        if (elementCodeList == null) {
            return EMPTY_INDEXATIONCODELIST;
        }
        int size = elementCodeList.size();
        if (size == 0) {
            return EMPTY_INDEXATIONCODELIST;
        }
        if (size == 1) {
            return new UniqueIndexationCodeList(elementCodeList.get(0), ponderationList.get(0));
        }
        int[] result = new int[size];
        float[] ponderationResult = new float[size];
        for (int i = 0; i < size; i++) {
            result[i] = elementCodeList.get(i);
            ponderationResult[i] = ponderationList.get(i);
        }
        return new ArrayIndexationCodeList(result, ponderationResult);
    }

    public static CodeList shuffle(CodeList codeList, int finalSize) {
        int count = codeList.getCodeCount();
        if (count < 2) {
            return codeList;
        }
        List<Integer> list = new ArrayList<Integer>();
        for (int i = 0; i < count; i++) {
            list.add(codeList.getCode(i));
        }
        Collections.shuffle(list);
        if ((finalSize == -1) || (count <= finalSize)) {
            return toCodeList(list);
        } else {
            int[] array = new int[finalSize];
            for (int i = 0; i < finalSize; i++) {
                array[i] = list.get(i);
            }
            return new ArrayCodeList(array);
        }
    }


    private static class EmptyCodeList implements CodeList {

        private EmptyCodeList() {
        }

        @Override
        public int getCodeCount() {
            return 0;
        }

        @Override
        public int getCode(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyElementCodeList implements ElementCodeList {

        private EmptyElementCodeList() {
        }

        @Override
        public int getCodeCount() {
            return 0;
        }

        @Override
        public int getCode(int i) {
            throw new IndexOutOfBoundsException();
        }

        @Override
        public int getSubsetCode(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class EmptyIndexationCodeList implements IndexationCodeList {

        private EmptyIndexationCodeList() {
        }

        @Override
        public int getCodeCount() {
            return 0;
        }

        @Override
        public int getCode(int i) {
            throw new IndexOutOfBoundsException();
        }

        @Override
        public float getPonderation(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class ArrayCodeList implements CodeList {

        private final int[] array;

        private ArrayCodeList(int[] array) {
            this.array = array;
        }

        @Override
        public int getCodeCount() {
            return array.length;
        }

        @Override
        public int getCode(int i) {
            return array[i];
        }

    }


    private static class ArrayElementCodeList extends ArrayCodeList implements ElementCodeList {

        private final int[] subsetCodeArray;

        private ArrayElementCodeList(int[] array, int[] subsetCodeArray) {
            super(array);
            this.subsetCodeArray = subsetCodeArray;
        }

        @Override
        public int getSubsetCode(int i) {
            return subsetCodeArray[i];
        }

    }


    private static class ArrayIndexationCodeList extends ArrayCodeList implements IndexationCodeList {

        private final float[] ponderationArray;

        private ArrayIndexationCodeList(int[] array, float[] ponderationArray) {
            super(array);
            this.ponderationArray = ponderationArray;
        }

        @Override
        public float getPonderation(int i) {
            return ponderationArray[i];
        }

    }


    private static class UniqueCodeList implements CodeList {

        private final int unique;

        private UniqueCodeList(int unique) {
            this.unique = unique;
        }

        @Override
        public int getCodeCount() {
            return 1;
        }

        @Override
        public int getCode(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return unique;
        }

    }


    private static class UniqueElementCodeList extends UniqueCodeList implements ElementCodeList {

        private final int subsetCode;

        private UniqueElementCodeList(int unique, int subsetCode) {
            super(unique);
            this.subsetCode = subsetCode;
        }

        @Override
        public int getSubsetCode(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return subsetCode;
        }

    }


    private static class UniqueIndexationCodeList extends UniqueCodeList implements IndexationCodeList {

        private final float ponderation;

        private UniqueIndexationCodeList(int unique, float ponderation) {
            super(unique);
            this.ponderation = ponderation;
        }

        @Override
        public float getPonderation(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException();
            }
            return ponderation;
        }

    }

}
