/* ScrutariLib_SearchEngine - Copyright (c) 2005-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine;

import java.util.List;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.primitives.FuzzyDate;
import net.scrutari.data.CorpusData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.LexieAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.searchengine.api.QId;
import net.scrutari.searchengine.api.operands.search.AllOperand;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.ComputedOptions;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.pertinence.FichePertinence;
import net.scrutari.searchengine.api.pertinence.MotclePertinence;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.InLangOccurrenceList;
import net.scrutari.searchengine.tools.SearchFilter;
import net.scrutari.searchengine.tools.lexieseek.LexieSeeker;
import net.scrutari.searchengine.tools.operands.search.SearchOperationUtils;
import net.scrutari.searchengine.tools.options.ComputedOptionsEngine;
import net.scrutari.searchengine.tools.pertinence.PertinenceUtils;
import net.scrutari.searchengine.tools.result.FicheSearchResultBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchSourceBuilder;
import net.scrutari.searchengine.tools.result.MotcleSearchResultBuilder;


/**
 *
 * @author Vincent Calame
 */
public final class FicheSearchEngine {

    private final static int[] EMPTY_ARRAY = new int[0];

    private FicheSearchEngine() {
    }

    public static FicheSearchResult search(ScrutariDB scrutariDB, SearchOperation searchOperation, SearchOptions searchOptions, GlobalSearchOptions globalSearchOptions, QId qId, boolean newSearch) {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        FicheSearchResultBuilder builder = new FicheSearchResultBuilder(FicheSearchSourceBuilder.build(qId, searchOperation.getCanonicalQ(), searchOptions.getSearchOptionsDef()), newSearch);
        Operand rootOperand = searchOperation.getRootOperand();
        if (rootOperand instanceof AllOperand) {
            ComputedOptions computedOptions = ComputedOptionsEngine.compute(scrutariDB, dataAccess, searchOperation, searchOptions, true, false);
            builder.setFicheMaximum(computedOptions.getFicheMaximum());
            allFiche(builder, scrutariDB, dataAccess, computedOptions, globalSearchOptions);
        } else {
            ComputedOptions computedOptions = ComputedOptionsEngine.compute(scrutariDB, dataAccess, searchOperation, searchOptions, true, true);
            builder.setFicheMaximum(computedOptions.getFicheMaximum());
            search(builder, scrutariDB, dataAccess, searchOperation, computedOptions, globalSearchOptions);
        }
        dataAccess.close();
        return builder.toFicheSearchResult();
    }

    private static FicheSearchResult search(FicheSearchResultBuilder builder, ScrutariDB scrutariDB, DataAccess dataAccess, SearchOperation searchOperation, ComputedOptions computedOptions, GlobalSearchOptions globalSearchOptions) {
        int operandMaxNumber = searchOperation.getSearchTokenOperandMaxNumber();
        int[] acceptOperandArray = SearchOperationUtils.toAcceptOperandArray(searchOperation);
        SearchEngine searchEngine = new SearchEngine(operandMaxNumber, acceptOperandArray);
        LexieAccess lexieAccess = scrutariDB.openLexieAccess();
        LexieSeeker lexieSeeker = new LexieSeeker(computedOptions, searchEngine, lexieAccess);
        lexieSeeker.seek(searchOperation);
        lexieAccess.close();
        if (computedOptions.getCodethesaurusArray() != null) {
            searchEngine.checkMotclePertinence(dataAccess, computedOptions);
        }
        FicheSort ficheSort = new FicheSort(globalSearchOptions, acceptOperandArray, computedOptions.getPertinencePonderation(), dataAccess);
        SearchFilter searchFilter = new SearchFilter(searchOperation);
        if (computedOptions.hasMandatoryOperand()) {
            List<FichePertinence> list = searchEngine.getFichePertinenceList();
            for (FichePertinence fichePertinence : list) {
                if (searchFilter.accept(fichePertinence)) {
                    FuzzyDate date = dataAccess.getFicheDate(fichePertinence.getCodefiche());
                    ficheSort.addFichePertinence(fichePertinence, date);
                }
            }
        } else {
            int[] codecorpusArray = computedOptions.getCodecorpusArray();
            int corpusCount = codecorpusArray.length;
            for (int i = 0; i < corpusCount; i++) {
                int codecorpus = codecorpusArray[i];
                CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
                int ficheCount = corpusData.getFicheCount();
                for (int j = 0; j < ficheCount; j++) {
                    int codefiche = corpusData.getCodeFiche(j);
                    if (computedOptions.acceptFiche(codefiche)) {
                        FichePertinence fichePertinence = searchEngine.getFichePertinence(codefiche);
                        if ((fichePertinence == null) || (searchFilter.accept(fichePertinence))) {
                            FuzzyDate date = dataAccess.getFicheDate(codefiche);
                            if (fichePertinence == null) {
                                ficheSort.addFichePertinence(codecorpus, codefiche, date);
                            } else {
                                ficheSort.addFichePertinence(fichePertinence, date);
                            }
                        }
                    }
                }
            }
        }
        ficheSort.initFicheSearchResultGroup(builder);
        List<MotclePertinence> mtclPertList = searchEngine.getMotclePertinenceList();
        builder.setMotcleCount(mtclPertList.size());
        for (MotclePertinence mtclPert : mtclPertList) {
            InLangOccurrenceList[] array = PertinenceUtils.getInLangOccurrenceListArray(mtclPert, acceptOperandArray);
            builder.addMotcleSearchResultInfo(MotcleSearchResultBuilder.toMotcleSearchResultInfo(mtclPert.getCodemotcle(), array));
        }
        return builder.toFicheSearchResult();
    }

    private static FicheSearchResult allFiche(FicheSearchResultBuilder builder, ScrutariDB scrutariDB, DataAccess dataAccess, ComputedOptions computedOptions, GlobalSearchOptions globalSearchOptions) {
        builder.setFicheMaximum(computedOptions.getFicheMaximum());
        FicheSort ficheSort = new FicheSort(globalSearchOptions, EMPTY_ARRAY, computedOptions.getPertinencePonderation(), dataAccess);
        int[] codecorpusArray = computedOptions.getCodecorpusArray();
        int length = codecorpusArray.length;
        for (int i = 0; i < length; i++) {
            int codecorpus = codecorpusArray[i];
            CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
            int ficheCount = corpusData.getFicheCount();
            for (int j = 0; j < ficheCount; j++) {
                int codefiche = corpusData.getCodeFiche(j);
                if (computedOptions.acceptFiche(codefiche)) {
                    FuzzyDate date = dataAccess.getFicheDate(codefiche);
                    ficheSort.addFichePertinence(codecorpus, codefiche, date);
                }
            }
        }
        ficheSort.initFicheSearchResultGroup(builder);
        return builder.toFicheSearchResult();
    }

}
