/* crutariLib_SearchEngine - Copyright (c) 2013-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import net.mapeadores.util.primitives.FuzzyDate;
import net.mapeadores.util.text.SubstringPosition;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.api.pertinence.FichePertinence;
import net.scrutari.searchengine.tools.pertinence.PertinencePack;
import net.scrutari.searchengine.tools.pertinence.PertinenceUtils;
import net.scrutari.searchengine.tools.result.FicheSearchResultBuilder;
import net.scrutari.searchengine.tools.result.FicheSearchResultGroupBuilder;


/**
 *
 * @author Vincent Calame
 */
class FicheSort {

    private final static int[] EMPTY_ARRAY = new int[0];
    private final GlobalSearchOptions globalSearchOptions;
    private int globalFicheCount = 0;
    private Group defaultGroup;
    private SortedMap<Integer, Group> groupMap;
    private final List<Group> groupList;
    private final FuzzyDate currentDate = FuzzyDate.current();
    private final int currentYear = currentDate.getYear();
    private final int[] acceptOperandArray;
    private final DataAccess dataAccess;

    FicheSort(GlobalSearchOptions globalSearchOptions, int[] acceptOperandArray, PertinencePonderation pertinencePonderation, DataAccess dataAccess) {
        this.globalSearchOptions = globalSearchOptions;
        this.acceptOperandArray = acceptOperandArray;
        this.dataAccess = dataAccess;
        if (pertinencePonderation == null) {
            pertinencePonderation = globalSearchOptions.getPertinencePonderation();
        }
        int categoryCount = globalSearchOptions.getCategoryCount();
        if (categoryCount == 0) {
            defaultGroup = new Group(0, pertinencePonderation);
            groupList = Collections.singletonList(defaultGroup);
        } else {
            groupMap = new TreeMap<Integer, Group>();
            for (int i = 0; i < categoryCount; i++) {
                int rank = globalSearchOptions.getCategoryByIndex(i).getCategoryDef().getRank();
                groupMap.put(rank, new Group(rank, pertinencePonderation));
            }
            groupList = initSortedlist(groupMap);
        }
    }

    void addFichePertinence(int codecorpus, int codefiche, FuzzyDate date) {
        addFichePertinence(new NoneFichePertinence(codefiche, codecorpus), date);
    }

    void addFichePertinence(FichePertinence fichePertinence, FuzzyDate date) {
        Group group = getGroup(fichePertinence.getCodecorpus());
        group.addFichePertinence(fichePertinence, date);
        globalFicheCount++;
    }

    void initFicheSearchResultGroup(FicheSearchResultBuilder builder) {
        builder.setFicheCount(globalFicheCount);
        if (globalFicheCount == 0) {
            return;
        }
        int size = groupList.size();
        for (int i = 0; i < size; i++) {
            initFicheSearchResultGroup(builder, groupList.get(i));
        }
    }

    private Group getGroup(int codecorpus) {
        if (defaultGroup != null) {
            return defaultGroup;
        }
        Category category = globalSearchOptions.getCategoryByCorpus(codecorpus);
        return groupMap.get(category.getCategoryDef().getRank());
    }

    private void initFicheSearchResultGroup(FicheSearchResultBuilder builder, Group group) {
        if (group.getFicheCount() == 0) {
            return;
        }
        SortedSet<PertinencePack> set = group.sort();
        FicheSearchResultGroupBuilder groupBuilder = new FicheSearchResultGroupBuilder();
        groupBuilder.setCategoryRank(group.getCategoryRank());
        groupBuilder.ensureInfoCapacity(group.getFicheCount());
        for (PertinencePack pertinencePack : set) {
            builder.addCodecorpus(pertinencePack.getFichePertinence().getCodecorpus());
            groupBuilder.addFicheSearchResultInfo(PertinenceUtils.toFicheSearchResultInfo(pertinencePack, acceptOperandArray));
        }
        builder.addFicheSearchResultGroup(groupBuilder.toFicheSearchResultGroup());
    }


    private class Group {

        private final float originPonderation;
        private final float datePonderation;
        private final float occurrencePonderation;
        private final int categoryRank;
        private SortedSet<PertinencePack> pertinencePackSet = new TreeSet<PertinencePack>();
        private int ficheCount = 0;

        private Group(int categoryRank, PertinencePonderation pertinencePonderation) {
            this.categoryRank = categoryRank;
            this.originPonderation = pertinencePonderation.getOriginPonderation();
            this.datePonderation = pertinencePonderation.getDatePonderation();
            this.occurrencePonderation = pertinencePonderation.getOccurrencePonderation();
        }

        private int getCategoryRank() {
            return categoryRank;
        }

        private int getFicheCount() {
            return ficheCount;
        }

        private void addFichePertinence(FichePertinence fichePertinence, FuzzyDate date) {
            ficheCount++;
            float[] scoreArray = new float[3];
            scoreArray[0] = occurrencePonderation * PertinenceUtils.getOccurrenceScore(fichePertinence, acceptOperandArray);
            scoreArray[1] = datePonderation * PertinenceUtils.getDateScore(date, currentYear);
            scoreArray[2] = 0;
            pertinencePackSet.add(new PertinencePack(scoreArray, fichePertinence, date));
        }

        private SortedSet<PertinencePack> sort() {
            if (originPonderation != 0) {
                pertinencePackSet = PertinenceUtils.addOriginScore(pertinencePackSet, originPonderation, dataAccess);
            }
            return pertinencePackSet;
        }

    }

    private static List<Group> initSortedlist(SortedMap<Integer, Group> groupMap) {
        List<Group> list = new ArrayList<Group>();
        list.addAll(groupMap.values());
        Collections.reverse(list);
        return list;
    }


    private static class NoneFichePertinence implements FichePertinence {

        private final int codefiche;
        private final int codecorpus;

        private NoneFichePertinence(int codefiche, int codecorpus) {
            this.codefiche = codefiche;
            this.codecorpus = codecorpus;
        }

        @Override
        public short getOperandState(int operandNumber) {
            return FichePertinence.NO_OCCURRENCE_STATE;
        }

        @Override
        public int getCodefiche() {
            return codefiche;
        }

        @Override
        public int getCodecorpus() {
            return codecorpus;
        }

        @Override
        public int getAlineaInteger(int operandNumber) {
            return -1;
        }

        @Override
        public SubstringPosition getSubstringPosition(int operandNumber) {
            return null;
        }

        @Override
        public int[] getAcceptCodemotcleArray() {
            return EMPTY_ARRAY;
        }

        @Override
        public float getMotcleScore(int operandNumber) {
            return 0;
        }

    }

}
