/* ScrutariLib_SearchEngine - Copyright (c) 2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.LinkedHashSet;
import java.util.Set;
import net.mapeadores.util.exceptions.SwitchException;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.searchengine.api.options.ListReduction;


/**
 *
 * @author Vincent Calame
 */
public class ListReductionBuilder {

    private final short type;
    private final Set<ScrutariDataURI> uriSet = new LinkedHashSet<ScrutariDataURI>();
    private boolean exclude;


    public ListReductionBuilder(short type) {
        this.type = type;
    }

    public short getType() {
        return type;
    }

    public void setExclude(boolean exclude) {
        this.exclude = exclude;
    }

    public boolean isEmpty() {
        return uriSet.isEmpty();
    }

    public void add(ScrutariDataURI scrutariDataURI) {
        if (scrutariDataURI.getType() != type) {
            throw new IllegalArgumentException("Type don't matching: " + type + " / " + scrutariDataURI.getType());
        }
        uriSet.add(scrutariDataURI);
    }

    public ListReduction toListReduction() {
        ScrutariDataURI[] array = uriSet.toArray(new ScrutariDataURI[uriSet.size()]);
        return new InternalListReduction(type, exclude, array);
    }

    public static ListReduction build(DataAccess dataAccess, short type, boolean exclude, CodeList codeList) {
        ListReductionBuilder builder = new ListReductionBuilder(type);
        builder.setExclude(exclude);
        int codeCount = codeList.getCodeCount();
        for (int i = 0; i < codeCount; i++) {
            int code = codeList.getCode(i);
            switch (type) {
                case ScrutariDataURI.BASEURI_TYPE:
                    BaseData baseData = dataAccess.getBaseDataByCode(code);
                    if (baseData != null) {
                        builder.uriSet.add(baseData.getBaseURI());
                    }
                    break;
                case ScrutariDataURI.CORPUSURI_TYPE:
                    CorpusData corpusData = dataAccess.getCorpusDataByCode(code);
                    if (corpusData != null) {
                        builder.uriSet.add(corpusData.getCorpusURI());
                    }
                    break;
                case ScrutariDataURI.THESAURUSURI_TYPE:
                    ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(code);
                    if (thesaurusData != null) {
                        builder.uriSet.add(thesaurusData.getThesaurusURI());
                    }
                    break;
                case ScrutariDataURI.FICHEURI_TYPE:
                    FicheData ficheData = dataAccess.getFicheDataByCode(code);
                    if (ficheData != null) {
                        builder.uriSet.add(ficheData.getFicheURI());
                    }
                    break;
                case ScrutariDataURI.MOTCLEURI_TYPE:
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(code);
                    if (motcleData != null) {
                        builder.uriSet.add(motcleData.getMotcleURI());
                    }
                    break;
                default:
                    throw new SwitchException("Unkown type: " + type);
            }
        }
        return builder.toListReduction();
    }


    private static class InternalListReduction implements ListReduction {

        private final short type;
        private final boolean exclude;
        private final ScrutariDataURI[] array;

        private InternalListReduction(short type, boolean exclude, ScrutariDataURI[] array) {
            this.type = type;
            this.exclude = exclude;
            this.array = array;
        }

        @Override
        public short getType() {
            return type;
        }

        @Override
        public boolean isExclude() {
            return exclude;
        }

        @Override
        public int getScrutariDataURICount() {
            return array.length;
        }

        @Override
        public ScrutariDataURI getScrutariDataURI(int index) {
            return array[index];
        }

    }

}
