/* ScrutariLib_SearchEngine - Copyright (c) 2005-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.logging.MessageHandler;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.URITreeProvider;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.codes.ElementCodeList;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.operands.eligibility.Eligibility;
import net.scrutari.searchengine.api.operands.eligibility.EligibilityOperand;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.api.result.FicheSearchResultHolder;
import net.scrutari.searchengine.tools.operands.eligibility.EligibilityCheck;
import net.scrutari.searchengine.tools.operands.eligibility.EligibilityCheckFactory;
import net.scrutari.searchengine.tools.operands.eligibility.EligibilityCheckParameters;


/**
 *
 *
 * @author Vincent Calame
 */
public final class SearchOptionsBuilder {

    public final static SearchOptions EMPTY_SEARCHOPTIONS = new InternalSearchOptions(SearchOptionsDefBuilder.EMPTY_SEARCHOPTIONSDEF, null, null, null, null);
    private final SearchOptionsDef searchOptionsDef;
    private Eligibility ficheEligibility;
    private Set<Integer> corpusSet = null;
    private Set<Integer> thesaurusSet = null;
    private Map<Integer, Integer> ficheMap = null;

    public SearchOptionsBuilder(SearchOptionsDef searchOptionsDef) {
        this.searchOptionsDef = searchOptionsDef;
    }

    public SearchOptionsDef getSearchOptionsDef() {
        return searchOptionsDef;
    }

    public void reduceFicheList(ElementCodeList ficheCodeList) {
        if (ficheCodeList == null) {
            return;
        }
        if ((ficheMap != null) && (ficheMap.isEmpty())) {
            return;
        }
        if ((corpusSet != null) && (corpusSet.isEmpty())) {
            ficheMap = Collections.emptyMap();
            return;
        }
        int ficheCount = ficheCodeList.getCodeCount();
        if (ficheCount == 0) {
            corpusSet = Collections.emptySet();
            ficheMap = Collections.emptyMap();
            return;
        }
        if (corpusSet == null) {
            corpusSet = new LinkedHashSet<Integer>();
            ficheMap = new LinkedHashMap<Integer, Integer>();
            for (int i = 0; i < ficheCount; i++) {
                int codecorpus = ficheCodeList.getSubsetCode(i);
                corpusSet.add(codecorpus);
                ficheMap.put(ficheCodeList.getCode(i), codecorpus);
            }
            return;
        }
        if (ficheMap == null) {
            ficheMap = new LinkedHashMap<Integer, Integer>();
            for (int i = 0; i < ficheCount; i++) {
                int codecorpus = ficheCodeList.getSubsetCode(i);
                if (corpusSet.contains(codecorpus)) {
                    ficheMap.put(ficheCodeList.getCode(i), codecorpus);
                }
            }
            return;
        }
        Set<Integer> newCorpusSet = new HashSet<Integer>();
        Map<Integer, Integer> newFicheMap = new LinkedHashMap<Integer, Integer>();
        for (int i = 0; i < ficheCount; i++) {
            int codecorpus = ficheCodeList.getSubsetCode(i);
            if (corpusSet.contains(codecorpus)) {
                int codefiche = ficheCodeList.getCode(i);
                if (ficheMap.containsKey(codefiche)) {
                    newCorpusSet.add(codecorpus);
                    newFicheMap.put(codefiche, codecorpus);
                }
            }
        }
        corpusSet = newCorpusSet;
        ficheMap = newFicheMap;
    }

    public void reduceCorpusList(CodeList corpusCodeList) {
        if (corpusCodeList == null) {
            return;
        }
        if ((corpusSet != null) && (corpusSet.isEmpty())) {
            return;
        }
        int corpusCount = corpusCodeList.getCodeCount();
        if (corpusCount == 0) {
            corpusSet = Collections.emptySet();
            if (ficheMap != null) {
                ficheMap = Collections.emptyMap();
            }
            return;
        }
        if (corpusSet == null) {
            corpusSet = new LinkedHashSet<Integer>();
            int count = corpusCodeList.getCodeCount();
            for (int i = 0; i < count; i++) {
                corpusSet.add(corpusCodeList.getCode(i));
            }
            return;
        }
        Set<Integer> newCorpusSet = new HashSet<Integer>();
        for (int i = 0; i < corpusCount; i++) {
            int code = corpusCodeList.getCode(i);
            if (corpusSet.contains(code)) {
                newCorpusSet.add(code);
            }
        }
        if (ficheMap != null) {
            for (Iterator<Map.Entry<Integer, Integer>> it = ficheMap.entrySet().iterator(); it.hasNext();) {
                Map.Entry<Integer, Integer> entry = it.next();
                if (!newCorpusSet.contains(entry.getValue())) {
                    it.remove();
                }
            }
        }
        corpusSet = newCorpusSet;
    }

    public void reduceThesaurusList(CodeList thesaurusCodeList) {
        if (thesaurusCodeList == null) {
            return;
        }
        if ((thesaurusSet != null) && (thesaurusSet.isEmpty())) {
            return;
        }
        int count = thesaurusCodeList.getCodeCount();
        if (count == 0) {
            thesaurusSet = Collections.emptySet();
            return;
        }
        if (thesaurusSet == null) {
            thesaurusSet = new LinkedHashSet<Integer>();
            for (int i = 0; i < count; i++) {
                thesaurusSet.add(thesaurusCodeList.getCode(i));
            }
            return;
        }
        Set<Integer> newThesaurusSet = new HashSet<Integer>();
        for (int i = 0; i < count; i++) {
            int code = thesaurusCodeList.getCode(i);
            if (thesaurusSet.contains(code)) {
                newThesaurusSet.add(code);
            }
        }
        thesaurusSet = newThesaurusSet;
    }

    public CodeList getCurrentCorpusList() {
        if (corpusSet != null) {
            return CodesUtils.toCodeList(corpusSet);
        } else {
            return null;
        }
    }

    public SearchOptions toSearchOptions() {
        ElementCodeList codeficheList = null;
        CodeList codecorpusList = null;
        CodeList codethesaurusList = null;
        if (ficheMap != null) {
            codeficheList = CodesUtils.toElementCodeList(ficheMap);
        }
        if (corpusSet != null) {
            codecorpusList = CodesUtils.toCodeList(corpusSet);
        }
        if (thesaurusSet != null) {
            codethesaurusList = CodesUtils.toCodeList(thesaurusSet);
        }
        return new InternalSearchOptions(searchOptionsDef, ficheEligibility, codeficheList, codecorpusList, codethesaurusList);
    }

    public void checkEligibility(DataAccess dataAccess, FicheSearchResultHolder ficheSearchResultHolder, MessageHandler messageHandler, URITreeProvider uriTreeProvider) {
        EligibilityCheckParameters eligibilityCheckParameters = new EligibilityCheckParameters()
                .setDataAccess(dataAccess)
                .setFicheSearchResultHolder(ficheSearchResultHolder)
                .setMessageHandler(messageHandler)
                .setUriTreeProvider(uriTreeProvider);
        EligibilityOperand ficheEligibilityOperand = searchOptionsDef.getFicheEligibilityOperand();
        if (ficheEligibilityOperand == null) {
            return;
        }
        EligibilityCheck eligibilityCheck = EligibilityCheckFactory.check(ficheEligibilityOperand, getCurrentCorpusList(), eligibilityCheckParameters);
        if (eligibilityCheck == null) {
            return;
        }
        Set<Integer> codeSet = eligibilityCheck.getCodeSet();
        if (codeSet != null) {
            if (corpusSet == null) {
                corpusSet = new HashSet<Integer>(codeSet);
            } else {
                corpusSet.retainAll(codeSet);
            }
        }
        this.ficheEligibility = eligibilityCheck.getEligibility();
    }

    public static SearchOptionsBuilder derive(SearchOptions searchOptions) {
        SearchOptionsBuilder builder = new SearchOptionsBuilder(searchOptions.getSearchOptionsDef());
        builder.reduceFicheList(searchOptions.getFicheCodeList());
        builder.reduceCorpusList(searchOptions.getCorpusCodeList());
        builder.reduceThesaurusList(searchOptions.getThesaurusCodeList());
        builder.ficheEligibility = searchOptions.getFicheEligibility();
        return builder;
    }


    private static class InternalSearchOptions implements SearchOptions {

        private final SearchOptionsDef searchOptionsDef;
        private final Eligibility ficheEligibility;
        private final ElementCodeList codeficheList;
        private final CodeList codecorpusList;
        private final CodeList codethesaurusList;


        private InternalSearchOptions(SearchOptionsDef searchOptionsDef, Eligibility ficheEligibility, ElementCodeList codeficheList, CodeList codecorpusList, CodeList codethesaurusList) {
            this.searchOptionsDef = searchOptionsDef;
            this.ficheEligibility = ficheEligibility;
            this.codeficheList = codeficheList;
            this.codecorpusList = codecorpusList;
            this.codethesaurusList = codethesaurusList;
        }

        @Override
        public SearchOptionsDef getSearchOptionsDef() {
            return searchOptionsDef;
        }

        @Override
        public Eligibility getFicheEligibility() {
            return ficheEligibility;
        }

        @Override
        public ElementCodeList getFicheCodeList() {
            return codeficheList;
        }

        @Override
        public CodeList getCorpusCodeList() {
            return codecorpusList;
        }

        @Override
        public CodeList getThesaurusCodeList() {
            return codethesaurusList;
        }

    }

}
