/* ScrutariLib_SearchEngine - Copyright (c) 2014-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.attr.AppendableAttributeHolderBuilder;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeHolder;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.mapeadores.util.localisation.LangListUtils;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.text.CleanedString;
import net.scrutari.searchengine.api.operands.eligibility.EligibilityOperand;
import net.scrutari.searchengine.api.options.ListReduction;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.tools.operands.eligibility.EligibilityOperandParser;


/**
 *
 * @author Vincent Calame
 */
public class SearchOptionsDefBuilder {

    public final static SearchOptionsDef EMPTY_SEARCHOPTIONSDEF = new InternalSearchOptionsDef(null, null, new HashMap<Short, ListReduction>(), null, null, AttributeUtils.EMPTY_ATTRIBUTEHOLDER);
    private final AppendableAttributeHolderBuilder attributeHolderBuilder = new AppendableAttributeHolderBuilder();
    private final List<EligibilityOperand> ficheOperandList = new ArrayList<EligibilityOperand>();
    private final Map<Short, ListReduction> listReductionMap = new HashMap<Short, ListReduction>();
    private final Set<Lang> filterLangSet = new LinkedHashSet<Lang>();
    private Lang langUi;
    private PertinencePonderation pertinencePonderation;

    public SearchOptionsDefBuilder() {
    }

    public void setLangUi(Lang langUi) {
        this.langUi = langUi;
    }

    public void addFilterLang(Lang lang) {
        filterLangSet.add(lang);
    }

    public boolean hasFilterLang() {
        return !filterLangSet.isEmpty();
    }

    public Lang reduceToUniqueLang() {
        if (filterLangSet.isEmpty()) {
            return null;
        }
        boolean first = true;
        Lang result = null;
        for (Iterator<Lang> it = filterLangSet.iterator(); it.hasNext();) {
            if (first) {
                result = it.next();
                first = false;
            } else {
                it.next();
                it.remove();
            }
        }
        return result;
    }

    public void setPertinencePonderation(PertinencePonderation pertinencePonderation) {
        this.pertinencePonderation = pertinencePonderation;
    }

    public void addFicheEligibilityOperand(EligibilityOperand operand) {
        if (operand != null) {
            ficheOperandList.add(operand);
        }
    }

    public void putListReduction(ListReduction listReduction) {
        short type = listReduction.getType();
        if (listReduction.getScrutariDataURICount() == 0) {
            listReductionMap.remove(type);
        } else {
            listReductionMap.put(type, listReduction);
        }
    }

    public SearchOptionsDef toSearchOptionsDef() {
        EligibilityOperand ficheRootOperand = EligibilityOperandParser.toEligibilityOperand(ficheOperandList, LogicalOperationConstants.INTERSECTION_OPERATOR);
        Map<Short, ListReduction> finalListReductionMap = new HashMap<Short, ListReduction>(listReductionMap);
        LangList filterLangList;
        if (filterLangSet.isEmpty()) {
            filterLangList = LangListUtils.EMPTY_LANGLIST;
        } else {
            filterLangList = LangListUtils.fromCollection(filterLangSet);
        }
        return new InternalSearchOptionsDef(langUi, filterLangList, finalListReductionMap, ficheRootOperand, pertinencePonderation, attributeHolderBuilder.toAttributeHolder());
    }

    public Attribute getAttribute(AttributeKey attributeKey) {
        return attributeHolderBuilder.getAttribute(attributeKey);
    }

    public void addAttribute(AttributeKey attributeKey, List<CleanedString> valueList) {
        attributeHolderBuilder.addAttribute(attributeKey, valueList);
    }

    public void addAttribute(AttributeKey attributeKey, CleanedString cleanedString) {
        attributeHolderBuilder.addAttribute(attributeKey, cleanedString);
    }

    public void addAttribute(Attribute attribute) {
        attributeHolderBuilder.addAttribute(attribute);
    }


    private static class InternalSearchOptionsDef implements SearchOptionsDef {

        private final Lang langUi;
        private final LangList filterLangList;
        private final Map<Short, ListReduction> listReductionMap;
        private final EligibilityOperand ficheEligibilityOperand;
        private final PertinencePonderation pertinencePonderation;
        private final AttributeHolder attributeHolder;

        private InternalSearchOptionsDef(Lang langUi, LangList filterLangList, Map<Short, ListReduction> listReductionMap, EligibilityOperand ficheEligibilityOperand, PertinencePonderation pertinencePonderation, AttributeHolder attributeHolder) {
            this.langUi = langUi;
            this.filterLangList = filterLangList;
            this.ficheEligibilityOperand = ficheEligibilityOperand;
            this.pertinencePonderation = pertinencePonderation;
            this.listReductionMap = listReductionMap;
            this.attributeHolder = attributeHolder;
        }

        @Override
        public Lang getLangUi() {
            return langUi;
        }

        @Override
        public LangList getFilterLangList() {
            return filterLangList;
        }

        @Override
        public EligibilityOperand getFicheEligibilityOperand() {
            return ficheEligibilityOperand;
        }

        @Override
        public PertinencePonderation getPertinencePonderation() {
            return pertinencePonderation;
        }

        @Override
        public ListReduction getListReduction(short type) {
            return listReductionMap.get(type);
        }

        @Override
        public Attribute getAttributeByKey(AttributeKey key) {
            return attributeHolder.getAttributeByKey(key);
        }

        @Override
        public int getAttributeCount() {
            return attributeHolder.getAttributeCount();
        }

        @Override
        public Attribute getAttributeByIndex(int i) {
            return attributeHolder.getAttributeByIndex(i);
        }

    }

}
