/* ScrutariLib_Supermotcle - Copyright (c) 2010-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.supermotcle.tools;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.collation.CollationUnit;
import net.scrutari.data.BaseData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.ElementCodeList;
import net.scrutari.db.api.codes.IndexationCodeList;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.supermotcle.api.Supermotcle;


/**
 *
 * @author Vincent Calame
 */
public class SupermotcleBuilder {

    private final static IndexationCounterComparator INDEXATION_COMPARATOR = new IndexationCounterComparator();
    private final Lang lang;
    private final int code;
    private final List<Integer> codemotcleList = new ArrayList<Integer>();
    private final List<Integer> codethesaurusList = new ArrayList<Integer>();
    private final DataAccess dataAccess;
    private final List<IndexationCounter> indexationCounterList = new ArrayList<IndexationCounter>();
    private CollationUnit collationUnit;

    public SupermotcleBuilder(DataAccess dataAccess, Lang lang, int code, int codethesaurus, int codemotcle, CollationUnit collationUnit) {
        this.dataAccess = dataAccess;
        this.lang = lang;
        this.code = code;
        this.collationUnit = collationUnit;
        codemotcleList.add(codemotcle);
        codethesaurusList.add(codethesaurus);
        int baseCount = dataAccess.getBaseCount();
        for (int i = 0; i < baseCount; i++) {
            BaseData baseData = dataAccess.getBaseDataByIndex(i);
            int codebase = baseData.getCodeBase();
            int corpusCount = baseData.getCorpusCount();
            int[] codecorpusArray = new int[corpusCount];
            for (int j = 0; j < corpusCount; j++) {
                codecorpusArray[j] = baseData.getCodeCorpus(j);
            }
            IndexationCounter indexationCounter = new IndexationCounter(codebase, codecorpusArray);
            indexationCounterList.add(indexationCounter);
        }
        countIndexation(codemotcle);
    }

    public void addMotcle(int codethesaurus, int codemotcle, CollationUnit newCollationUnit) { //On compare pour voir quelle est la meilleur « collation »
        String oldSourceString = collationUnit.getSourceString();
        String newSourceString = newCollationUnit.getSourceString();
        boolean replace = false;
        int newLength = newSourceString.length();
        int oldLength = oldSourceString.length();
        if (newLength < oldLength) {
            replace = true;
        } else if (newLength == oldLength) {
            if (newSourceString.compareTo(oldSourceString) > 0) {
                replace = true;
            }
        }
        if (replace) {
            collationUnit = newCollationUnit;
        }
        codemotcleList.add(codemotcle);
        codethesaurusList.add(codethesaurus);
        countIndexation(codemotcle);
    }

    public Supermotcle toSupermotcle() {
        InternalSupermotcle is = new InternalSupermotcle(this);
        return is;
    }

    private void countIndexation(int codemotcle) {
        int baseCount = indexationCounterList.size();
        for (int i = 0; i < baseCount; i++) {
            IndexationCounter indexationCounter = indexationCounterList.get(i);
            indexationCounter.checkMotcle(codemotcle);
        }
    }


    private class IndexationCounter {

        private final int codebase;
        private final int[] codecorpusArray;
        private int indexationCount;

        private IndexationCounter(int codebase, int[] codecorpusArray) {
            this.codebase = codebase;
            this.codecorpusArray = codecorpusArray;
            this.indexationCount = 0;
        }

        private void checkMotcle(int codemotcle) {
            int length = codecorpusArray.length;
            for (int i = 0; i < length; i++) {
                IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, codecorpusArray[i]);
                int ficheCount = indexationCodeList.getCodeCount();
                indexationCount = indexationCount + ficheCount;
            }
        }

    }


    private static class InternalSupermotcle implements Supermotcle {

        private final Lang lang;
        private final int code;
        private final CollationUnit collationUnit;
        private final ElementCodeList codemotcleList;
        private final int indexationCount;
        private final int[] codeBaseArray;
        private final int[] indexationCountByBaseArray;

        private InternalSupermotcle(SupermotcleBuilder builder) {
            this.lang = builder.lang;
            this.code = builder.code;
            this.collationUnit = builder.collationUnit;
            this.codemotcleList = CodesUtils.toElementCodeList(builder.codemotcleList, builder.codethesaurusList);
            Set<IndexationCounter> resultSet = new TreeSet<IndexationCounter>(INDEXATION_COMPARATOR);
            int indexCount = 0;
            for (IndexationCounter indexationCounter : builder.indexationCounterList) {
                if (indexationCounter.indexationCount > 0) {
                    indexCount = indexCount + indexationCounter.indexationCount;
                    resultSet.add(indexationCounter);
                }
            }
            this.indexationCount = indexCount;
            int baseCount = resultSet.size();
            codeBaseArray = new int[baseCount];
            indexationCountByBaseArray = new int[baseCount];
            int p = 0;
            for (IndexationCounter indexationCounter : resultSet) {
                codeBaseArray[p] = indexationCounter.codebase;
                indexationCountByBaseArray[p] = indexationCounter.indexationCount;
                p++;
            }
        }

        @Override
        public Lang getLang() {
            return lang;
        }

        @Override
        public int getCode() {
            return code;
        }

        @Override
        public CollationUnit getCollationUnit() {
            return collationUnit;
        }

        @Override
        public ElementCodeList getCodemotcleList() {
            return codemotcleList;
        }

        @Override
        public int getBaseCount() {
            return codeBaseArray.length;
        }

        @Override
        public int getCodeBase(int i) {
            return codeBaseArray[i];
        }

        @Override
        public int getIndexationCountByBase(int i) {
            return indexationCountByBaseArray[i];
        }

        @Override
        public int getIndexationCount() {
            return indexationCount;
        }

    }


    private static class IndexationCounterComparator implements Comparator<IndexationCounter> {

        private IndexationCounterComparator() {
        }

        @Override
        public int compare(IndexationCounter s1, IndexationCounter s2) {
            int ic1 = s1.indexationCount;
            int ic2 = s2.indexationCount;
            if (ic1 > ic2) {
                return -1;
            }
            if (ic1 < ic2) {
                return 1;
            }
            int c1 = s1.codebase;
            int c2 = s2.codebase;
            if (c1 < c2) {
                return -1;
            }
            if (c2 > c1) {
                return 1;
            }
            return 0;
        }

    }

}
