/* SctServer - Copyright (c) 2010-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.json.FicheDataJson;
import fr.exemole.sctserver.json.JsonUtils;
import fr.exemole.sctserver.json.MotcleDataJson;
import fr.exemole.sctserver.request.FicheFieldsFilter;
import fr.exemole.sctserver.request.IntitulesFilter;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.tools.util.BaseCheck;
import net.scrutari.db.tools.util.DataLabelUtils;
import net.scrutari.searchengine.api.CanonicalQ;
import net.scrutari.searchengine.api.QId;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.CategoryDef;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.FicheSearchSource;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;
import net.scrutari.searchengine.tools.options.OptionsUtils;


/**
 *
 * @author vic
 */
public class FicheSearchResultJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariDB scrutariDB;
    private final AttributeKeyCoder attributeKeyCoder;
    private final FicheSearchResult searchResult;
    private final FicheFieldsFilter ficheFieldsFilter;
    private final MotcleFieldsFilter motcleFieldsFilter;
    private final IntitulesFilter intitulesFilter;
    private final GlobalSearchOptions globalSearchOptions;
    private final Lang langUi;
    private final StartParameters startParameters;

    public FicheSearchResultJsonProducer(int version, FicheSearchResult searchResult, ScrutariDB scrutariDB, GlobalSearchOptions globalSearchOptions, FicheFieldsFilter ficheFieldsFilter, MotcleFieldsFilter motcleFieldsFilter, IntitulesFilter intitulesFilter, StartParameters startParameters, Lang langUi) {
        if (langUi == null) {
            throw new IllegalArgumentException("langUi is null");
        }
        this.version = version;
        this.searchResult = searchResult;
        this.ficheFieldsFilter = ficheFieldsFilter;
        this.motcleFieldsFilter = motcleFieldsFilter;
        this.intitulesFilter = intitulesFilter;
        this.startParameters = startParameters;
        this.scrutariDB = scrutariDB;
        this.globalSearchOptions = globalSearchOptions;
        this.langUi = langUi;
        this.attributeKeyCoder = scrutariDB.getAttributeKeyCoder();
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        int categoryCount = globalSearchOptions.getCategoryCount();
        FicheSearchSource ficheSearchSource = searchResult.getFicheSearchSource();
        CanonicalQ canonicalQ = ficheSearchSource.getCanonicalQ();
        int motcleCount = searchResult.getMotcleCount();
        Set<Integer> codecorpusSet = new LinkedHashSet<Integer>();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        jsonWriter.key("ficheSearchResult");
        jsonWriter.object();
        jsonWriter.key("q");
        jsonWriter.value(canonicalQ.toString());
        QId qId = ficheSearchSource.getQId();
        if (qId != null) {
            jsonWriter.key("qId");
            jsonWriter.value(qId.toString());
        }
        if (version < 2) {
            jsonWriter.key("qSource");
            jsonWriter.value(canonicalQ.toString());
        }
        jsonWriter.key("langUi");
        jsonWriter.value(langUi.toString());
        jsonWriter.key("ficheCount");
        jsonWriter.value(searchResult.getFicheCount());
        jsonWriter.key("ficheMaximum");
        jsonWriter.value(searchResult.getFicheMaximum());
        jsonWriter.key("motcleCount");
        jsonWriter.value(motcleCount);
        jsonWriter.key("ficheGroupType");
        if (categoryCount > 0) {
            if (version == 0) {
                jsonWriter.value("corpusclasse");
            } else {
                jsonWriter.value("category");
            }
        } else {
            jsonWriter.value("none");
        }

        jsonWriter.key("ficheGroupArray");
        jsonWriter.array();
        switch (startParameters.getType()) {
            case StartParameters.NONE_TYPE:
                writeNoneStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.GLOBAL_TYPE:
                writeGlobalStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.IN_ALL_TYPE:
                writeInAllStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.IN_SELECTION_TYPE:
                writeInSelectionStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
        }
        jsonWriter.endArray();
        jsonWriter.key("motcleArray");
        jsonWriter.array();
        for (int i = 0; i < motcleCount; i++) {
            MotcleSearchResultInfo motcleSearchResultInfo = searchResult.getMotcleResult(i);
            MotcleData motcleData = dataAccess.getMotcleDataByCode(motcleSearchResultInfo.getCode());
            jsonWriter.object();
            MotcleDataJson.object(jsonWriter, motcleData, motcleFieldsFilter, motcleSearchResultInfo, langUi);
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
        if (intitulesFilter != null) {
            boolean withComplement = (intitulesFilter.isWithComplement() && ((ficheFieldsFilter.isWithComplements()) || (ficheFieldsFilter.isWithMComplements())));
            boolean withCorpus = intitulesFilter.isWithCorpus();
            if (withCorpus || withComplement) {
                jsonWriter.key("corpusIntituleArray");
                jsonWriter.array();
                for (int codecorpus : codecorpusSet) {
                    CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
                    CorpusURI corpusURI = corpusData.getCorpusURI();
                    String corpusName = corpusURI.getCorpusName();
                    jsonWriter.object();
                    jsonWriter.key("codecorpus");
                    jsonWriter.value(codecorpus);
                    if (withCorpus) {
                        jsonWriter.key("corpus");
                        jsonWriter.value(DataLabelUtils.checkLabelString(corpusData, CorpusData.INTITULECORPUS_FIELDNUMBER, langUi, corpusName));
                        jsonWriter.key("fiche");
                        jsonWriter.value(DataLabelUtils.checkLabelString(corpusData, CorpusData.INTITULEFICHE_FIELDNUMBER, langUi, corpusName));
                    }
                    if (withComplement) {
                        int compMaxNumber = corpusData.getComplementMaxNumber();
                        for (int num = 1; num <= compMaxNumber; num++) {
                            jsonWriter.key("complement_" + num);
                            jsonWriter.value(DataLabelUtils.checkLabelString(corpusData, num, langUi, "complement_" + num));
                        }
                    }
                    jsonWriter.endObject();
                }
                jsonWriter.endArray();
            }
            if ((version > 0) && (intitulesFilter.isWithAttributes())) {
                JsonUtils.addAttributes(jsonWriter, attributeKeyCoder, langUi, version);
            }
        }
        if ((version == 0) && (ficheFieldsFilter.getMAttributesFilter() != FicheFieldsFilter.NONE_FILTER)) {
            JsonUtils.addAttributes(jsonWriter, attributeKeyCoder, langUi, version);
        }
        jsonWriter.endObject();
        writeWarnings(jsonWriter);
        jsonWriter.endObject();
        dataAccess.close();
    }


    private void writeNoneStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
            startGroup(jsonWriter, ficheSearchResultGroup);
            int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            for (int j = 0; j < ficheByGroupCount; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
            endGroup(jsonWriter);
        }
    }

    private void writeGlobalStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int globalStartIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        boolean limitStop = false;
        int ficheIncrement = 0;
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
            int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            int startIndex = 0;
            if (ficheIncrement == 0) {
                if (ficheByGroupCount < globalStartIndex) {
                    globalStartIndex = globalStartIndex - ficheByGroupCount;
                    continue;
                } else {
                    startIndex = globalStartIndex;
                }
            }
            startGroup(jsonWriter, ficheSearchResultGroup);
            for (int j = startIndex; j < ficheByGroupCount; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
                if (withLimit) {
                    ficheIncrement++;
                    if (ficheIncrement == limit) {
                        limitStop = true;
                        break;
                    }
                }
            }
            endGroup(jsonWriter);
            if (limitStop) {
                break;
            }
        }
    }

    private void writeInAllStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int startIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
            int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            if (ficheByGroupCount < startIndex) {
                continue;
            }
            startGroup(jsonWriter, ficheSearchResultGroup);
            int min = ficheByGroupCount;
            if (withLimit) {
                min = Math.min(ficheByGroupCount, startIndex + limit);
            }
            for (int j = startIndex; j < min; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
            endGroup(jsonWriter);
        }
    }

    private void writeInSelectionStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int startIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
            if (!startParameters.containsCategory(ficheSearchResultGroup.getCategoryRank())) {
                continue;
            }
            int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            if (ficheByGroupCount < startIndex) {
                continue;
            }

            startGroup(jsonWriter, ficheSearchResultGroup);
            int min = ficheByGroupCount;
            if (withLimit) {
                min = Math.min(ficheByGroupCount, startIndex + limit);
            }
            for (int j = startIndex; j < min; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
            endGroup(jsonWriter);
        }
    }

    private void startGroup(JSONWriter jsonWriter, FicheSearchResultGroup ficheSearchResultGroup) throws IOException {
        jsonWriter.object();
        if (globalSearchOptions.getCategoryCount() > 0) {
            if (version == 0) {
                jsonWriter.key("corpusClasse");
            } else {
                jsonWriter.key("category");
            }
            jsonWriter.object();
            int categoryRank = ficheSearchResultGroup.getCategoryRank();
            jsonWriter.key("rank");
            jsonWriter.value(categoryRank);
            Category category = globalSearchOptions.getCategoryByRank(categoryRank);
            if (category != null) {
                CategoryDef categoryDef = category.getCategoryDef();
                jsonWriter.key("name");
                jsonWriter.value(categoryDef.getName());
                if (version == 0) {
                    jsonWriter.key("intitule");
                } else {
                    jsonWriter.key("title");
                }
                jsonWriter.value(OptionsUtils.checkLabelString(categoryDef, langUi, categoryDef.getName()));
            } else {
                jsonWriter.key("name");
                jsonWriter.value("unknown_rank_" + categoryRank);
                if (version == 0) {
                    jsonWriter.key("intitule");
                } else {
                    jsonWriter.key("title");
                }
                jsonWriter.key("## Unknown Rank : " + categoryRank + " ##");
            }
            jsonWriter.endObject();
        }
        jsonWriter.key("ficheCount");
        jsonWriter.value(ficheSearchResultGroup.getFicheSearchResultInfoCount());
        jsonWriter.key("ficheArray");
        jsonWriter.array();
    }

    private void endGroup(JSONWriter jsonWriter) throws IOException {
        jsonWriter.endArray();
        jsonWriter.endObject();
    }

    private void writeFicheSearchResultInfo(JSONWriter jsonWriter, FicheSearchResultInfo ficheSearchResultInfo, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int codefiche = ficheSearchResultInfo.getCode();
        FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
        codecorpusSet.add(ficheData.getCodeCorpus());
        BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap);
        jsonWriter.object();
        FicheDataJson.object(jsonWriter, ficheData, ficheFieldsFilter, baseCheck, ficheSearchResultInfo, attributeKeyCoder);
        int ficheMotcleCount = ficheSearchResultInfo.getMotcleCount();
        if (ficheMotcleCount > 0) {
            jsonWriter.key("codemotcleArray");
            jsonWriter.array();
            for (int k = 0; k < ficheMotcleCount; k++) {
                jsonWriter.value(ficheSearchResultInfo.getMotcleCode(k));
            }
            jsonWriter.endArray();
        }
        jsonWriter.endObject();
    }

    private BaseCheck getBaseCheck(DataAccess dataAccess, int codebase, Map<Integer, BaseCheck> bufferMap) {
        BaseCheck baseCheck = bufferMap.get(codebase);
        if (baseCheck != null) {
            return baseCheck;
        }
        BaseData baseData = dataAccess.getBaseDataByCode(codebase);
        baseCheck = BaseCheck.check(baseData, langUi);
        bufferMap.put(codebase, baseCheck);
        return baseCheck;
    }

}
