/* SctServer - Copyright (c) 2015-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.json.FicheDataJson;
import fr.exemole.sctserver.json.MotcleDataJson;
import fr.exemole.sctserver.request.FicheFieldsFilter;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.BaseData;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.tools.util.BaseCheck;
import net.scrutari.searchengine.api.QId;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.api.result.FicheSearchSource;
import net.scrutari.searchengine.api.result.MotcleSearchResultInfo;


/**
 *
 * @author Vincent Calame
 */
public class GeoJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariDB scrutariDB;
    private final AttributeKeyCoder attributeKeyCoder;
    private final FicheSearchResult searchResult;
    private final FicheFieldsFilter ficheFieldsFilter;
    private final MotcleFieldsFilter motcleFieldsFilter;
    private final GlobalSearchOptions globalSearchOptions;
    private final Lang langUi;
    private final StartParameters startParameters;

    public GeoJsonProducer(int version, FicheSearchResult searchResult, ScrutariDB scrutariDB, GlobalSearchOptions globalSearchOptions, FicheFieldsFilter ficheFieldsFilter, MotcleFieldsFilter motcleFieldsFilter, StartParameters startParameters, Lang langUi) {
        this.version = version;
        this.searchResult = searchResult;
        this.ficheFieldsFilter = ficheFieldsFilter;
        this.motcleFieldsFilter = motcleFieldsFilter;
        this.startParameters = startParameters;
        this.scrutariDB = scrutariDB;
        this.globalSearchOptions = globalSearchOptions;
        this.langUi = langUi;
        this.attributeKeyCoder = scrutariDB.getAttributeKeyCoder();
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        int motcleCount = searchResult.getMotcleCount();
        Set<Integer> codecorpusSet = new LinkedHashSet<Integer>();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        jsonWriter.key("type");
        jsonWriter.value("FeatureCollection");
        jsonWriter.key("features");
        jsonWriter.array();
        switch (startParameters.getType()) {
            case StartParameters.NONE_TYPE:
                writeNoneStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.GLOBAL_TYPE:
                writeGlobalStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.IN_ALL_TYPE:
                writeInAllStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
            case StartParameters.IN_SELECTION_TYPE:
                writeInSelectionStart(jsonWriter, dataAccess, codecorpusSet, bufferMap);
                break;
        }
        jsonWriter.endArray();
        FicheSearchSource ficheSearchSource = searchResult.getFicheSearchSource();
        jsonWriter.key("searchMetadata");
        jsonWriter.object();
        jsonWriter.key("q");
        jsonWriter.value(ficheSearchSource.getCanonicalQ().toString());
        QId qId = ficheSearchSource.getQId();
        if (qId != null) {
            jsonWriter.key("qId");
            jsonWriter.value(qId.toString());
        }
        jsonWriter.key("langUi");
        jsonWriter.value(langUi.toString());
        jsonWriter.key("ficheCount");
        jsonWriter.value(searchResult.getFicheCount());
        jsonWriter.key("ficheMaximum");
        jsonWriter.value(searchResult.getFicheMaximum());
        jsonWriter.key("motcleCount");
        jsonWriter.value(motcleCount);
        jsonWriter.key("motcleArray");
        jsonWriter.array();
        for (int i = 0; i < motcleCount; i++) {
            MotcleSearchResultInfo motcleSearchResultInfo = searchResult.getMotcleResult(i);
            MotcleData motcleData = dataAccess.getMotcleDataByCode(motcleSearchResultInfo.getCode());
            jsonWriter.object();
            MotcleDataJson.object(jsonWriter, motcleData, motcleFieldsFilter, motcleSearchResultInfo, langUi);
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
        jsonWriter.endObject();
        jsonWriter.endObject();
        dataAccess.close();
    }

    private void writeNoneStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
            int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            for (int j = 0; j < ficheByGroupCount; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
        }
    }

    private void writeGlobalStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int globalStartIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        boolean limitStop = false;
        int ficheIncrement = 0;
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
            int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            int startIndex = 0;
            if (ficheIncrement == 0) {
                if (ficheByGroupCount < globalStartIndex) {
                    globalStartIndex = globalStartIndex - ficheByGroupCount;
                    continue;
                } else {
                    startIndex = globalStartIndex;
                }
            }
            for (int j = startIndex; j < ficheByGroupCount; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
                if (withLimit) {
                    ficheIncrement++;
                    if (ficheIncrement == limit) {
                        limitStop = true;
                        break;
                    }
                }
            }
            if (limitStop) {
                break;
            }
        }
    }

    private void writeInAllStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int startIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
            int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            if (ficheByGroupCount < startIndex) {
                continue;
            }
            int min = ficheByGroupCount;
            if (withLimit) {
                min = Math.min(ficheByGroupCount, startIndex + limit);
            }
            for (int j = startIndex; j < min; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
        }
    }

    private void writeInSelectionStart(JSONWriter jsonWriter, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int startIndex = startParameters.getStart() - 1;
        int limit = startParameters.getLimit();
        boolean withLimit = (limit > 0);
        int groupCount = searchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup ficheSearchResultGroup = searchResult.getFicheSearchResultGroup(i);
            if (!startParameters.containsCategory(ficheSearchResultGroup.getCategoryRank())) {
                continue;
            }
            int ficheByGroupCount = ficheSearchResultGroup.getFicheSearchResultInfoCount();
            if (ficheByGroupCount < startIndex) {
                continue;
            }
            int min = ficheByGroupCount;
            if (withLimit) {
                min = Math.min(ficheByGroupCount, startIndex + limit);
            }
            for (int j = startIndex; j < min; j++) {
                FicheSearchResultInfo ficheSearchResultInfo = ficheSearchResultGroup.getFicheSearchResultInfo(j);
                writeFicheSearchResultInfo(jsonWriter, ficheSearchResultInfo, dataAccess, codecorpusSet, bufferMap);
            }
        }
    }

    private void writeFicheSearchResultInfo(JSONWriter jsonWriter, FicheSearchResultInfo ficheSearchResultInfo, DataAccess dataAccess, Set<Integer> codecorpusSet, Map<Integer, BaseCheck> bufferMap) throws IOException {
        int codefiche = ficheSearchResultInfo.getCode();
        FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
        if (!ficheData.isWithGeo()) {
            return;
        }
        codecorpusSet.add(ficheData.getCodeCorpus());
        BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap);
        jsonWriter.object();
        jsonWriter.key("type");
        jsonWriter.value("Feature");
        jsonWriter.key("id");
        jsonWriter.value(codefiche);
        jsonWriter.key("geometry");
        jsonWriter.object();
        jsonWriter.key("type");
        jsonWriter.value("Point");
        jsonWriter.key("coordinates");
        jsonWriter.array();
        jsonWriter.value(ficheData.getLongitude().toString());
        jsonWriter.value(ficheData.getLatitude().toString());
        jsonWriter.endArray();
        jsonWriter.endObject();
        jsonWriter.key("properties");
        jsonWriter.object();
        FicheDataJson.object(jsonWriter, ficheData, ficheFieldsFilter, baseCheck, ficheSearchResultInfo, attributeKeyCoder);
        int ficheMotcleCount = ficheSearchResultInfo.getMotcleCount();
        if (ficheMotcleCount > 0) {
            jsonWriter.key("codemotcleArray");
            jsonWriter.array();
            for (int k = 0; k < ficheMotcleCount; k++) {
                jsonWriter.value(ficheSearchResultInfo.getMotcleCode(k));
            }
            jsonWriter.endArray();
        }
        jsonWriter.endObject();
        jsonWriter.endObject();
    }

    private BaseCheck getBaseCheck(DataAccess dataAccess, int codebase, Map<Integer, BaseCheck> bufferMap) {
        BaseCheck baseCheck = bufferMap.get(codebase);
        if (baseCheck != null) {
            return baseCheck;
        }
        BaseData baseData = dataAccess.getBaseDataByCode(codebase);
        baseCheck = BaseCheck.check(baseData, langUi);
        bufferMap.put(codebase, baseCheck);
        return baseCheck;
    }

}
