/* SctServer - Copyright (c) 2010-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.api.EngineContext;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.context.EngineGroup;
import fr.exemole.sctserver.request.CountFilter;
import fr.exemole.sctserver.request.DataFilter;
import fr.exemole.sctserver.request.FicheFieldsFilter;
import fr.exemole.sctserver.request.IntitulesFilter;
import fr.exemole.sctserver.request.MotcleFieldsFilter;
import fr.exemole.sctserver.request.ParameterErrorMessageKeys;
import fr.exemole.sctserver.request.Parameters;
import fr.exemole.sctserver.request.RequestMapUtils;
import fr.exemole.sctserver.request.WarningHandler;
import fr.exemole.sctserver.request.WarningKeys;
import fr.exemole.sctserver.tools.EngineUtils;
import fr.exemole.sctserver.tools.log.SctLogUtils;
import java.text.ParseException;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.MotcleSearchEngine;
import net.scrutari.searchengine.SupermotcleSearchEngine;
import net.scrutari.searchengine.api.QId;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.MotcleSearchResult;
import net.scrutari.searchengine.api.result.SupermotcleSearchResult;
import net.scrutari.searchengine.tools.options.OptionsUtils;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;
import net.scrutari.searchengine.tools.options.SearchOptionsDefBuilder;


/**
 *
 * @author Vincent Calame
 */
public final class JsonProducerFactory {

    private JsonProducerFactory() {

    }

    public static JsonProducer getJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap) {
        WarningHandler warningHandler = new WarningHandler();
        AbstractJsonProducer abstractJsonProducer;
        try {
            abstractJsonProducer = getJsonProducer(scrutariSession, requestMap, warningHandler);
        } catch (ErrorMessageException eme) {
            Lang langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
            abstractJsonProducer = new ErrorJsonProducer(eme.getErrorMessage(), EngineUtils.getMessageLocalisation(scrutariSession.getEngine(), langUi));
        }
        if (!warningHandler.isEmpty()) {
            String warnings = requestMap.getParameter(Parameters.WARNINGS);
            if ((warnings != null) && (warnings.equals("1"))) {
                abstractJsonProducer.setWarningHandler(warningHandler);
            }
        }
        return abstractJsonProducer;
    }

    private static AbstractJsonProducer getJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) throws ErrorMessageException {
        String type = RequestMapUtils.getMandatoryParam(Parameters.TYPE, requestMap);
        String versionString = requestMap.getParameter(Parameters.VERSION);
        int version = 0;
        if (versionString != null) {
            version = -1;
            try {
                version = Integer.parseInt(versionString);
            } catch (NumberFormatException nfe) {
                throw new ErrorMessageException(ParameterErrorMessageKeys.WRONG_PARAMETER_VALUE, Parameters.VERSION, versionString);
            }
            if (version < 0) {
                throw new ErrorMessageException(ParameterErrorMessageKeys.UNKNOWN_PARAMETER_VALUE, Parameters.VERSION, versionString);
            }
        }
        if (type.equals("motcle")) {
            return getMotcleJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("q-motcle")) {
            return getMotcleSearchResultJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("q-supermotcle")) {
            return getSupermotcleSearchResultJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("base")) {
            return getBaseJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("fiche")) {
            return getFicheJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("q-fiche")) {
            return getFicheSearchResultJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("geojson")) {
            return getGeoJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("corpus")) {
            return getCorpusJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("thesaurus")) {
            return getThesaurusJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("c-fiche")) {
            return getFicheCountJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("category")) {
            return getCategoryJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("lang")) {
            return getLangJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("engine")) {
            return getEngineInfoJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else if (type.equals("enginegroup")) {
            return getEngineGroupJsonProducer(scrutariSession, requestMap, version, warningHandler);
        } else {
            throw new ErrorMessageException(ParameterErrorMessageKeys.UNKNOWN_PARAMETER_VALUE, Parameters.TYPE, type);
        }
    }

    private static AbstractJsonProducer getSupermotcleSearchResultJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) throws ErrorMessageException {
        SearchOperation searchOperation = RequestMapUtils.getSearchOperation(requestMap, scrutariSession.getScrutariDB().getAttributeKeyCoder(), warningHandler);
        Lang langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        SearchOptionsDefBuilder searchOptionsDefBuilder = RequestMapUtils.buildSearchOptionsDefBuilder(requestMap, warningHandler);
        searchOptionsDefBuilder.addFilterLang(langUi);
        int limit = RequestMapUtils.getLimit(requestMap, warningHandler);
        SupermotcleSearchResult searchResult = SupermotcleSearchEngine.search(scrutariSession.getSupermotcleProvider(), searchOperation, searchOptionsDefBuilder.toSearchOptionsDef());
        return new SupermotcleSearchResultJsonProducer(version, searchResult, langUi, limit);
    }

    private static AbstractJsonProducer getMotcleSearchResultJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) throws ErrorMessageException {
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        SearchOperation searchOperation = RequestMapUtils.getSearchOperation(requestMap, scrutariDB.getAttributeKeyCoder(), warningHandler);
        SearchOptionsDefBuilder searchOptionsDefBuilder = RequestMapUtils.buildSearchOptionsDefBuilder(requestMap, warningHandler);
        if (!searchOptionsDefBuilder.hasFilterLang()) {
            //@todo message d'erreur
            return null;
        }
        Lang lang = searchOptionsDefBuilder.reduceToUniqueLang();
        int limit = RequestMapUtils.getLimit(requestMap, warningHandler);
        SearchOptionsBuilder searchOptionsBuilder = new SearchOptionsBuilder(searchOptionsDefBuilder.toSearchOptionsDef());
        MotcleSearchResult searchResult = MotcleSearchEngine.search(scrutariDB, searchOperation, searchOptionsBuilder.toSearchOptions());
        return new MotcleSearchResultJsonProducer(version, searchResult, scrutariDB, lang, limit);

    }

    private static AbstractJsonProducer getFicheSearchResultJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) throws ErrorMessageException {
        FicheSearchResult searchResult = buildFicheSearchResult(scrutariSession, requestMap, warningHandler);
        FicheFieldsFilter ficheFieldsFilter = RequestMapUtils.getFicheFieldsFilter(requestMap);
        MotcleFieldsFilter motcleFieldsFilter = RequestMapUtils.getMotcleFieldsFilter(requestMap);
        IntitulesFilter intitulesFilter = RequestMapUtils.getIntitulesFilter(requestMap);
        StartParameters startParameters = RequestMapUtils.getStartParameters(requestMap, scrutariSession.getGlobalSearchOptions(), warningHandler);
        Lang langUi = searchResult.getFicheSearchSource().getSearchOptionsDef().getLangUi();
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        return new FicheSearchResultJsonProducer(version, searchResult, scrutariSession.getScrutariDB(), scrutariSession.getGlobalSearchOptions(), ficheFieldsFilter, motcleFieldsFilter, intitulesFilter, startParameters, langUi);
    }

    private static AbstractJsonProducer getGeoJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) throws ErrorMessageException {
        OptionalQRequestMap optionalQRequestMap = new OptionalQRequestMap(requestMap);
        FicheSearchResult searchResult = buildFicheSearchResult(scrutariSession, optionalQRequestMap, warningHandler);
        FicheFieldsFilter ficheFieldsFilter = RequestMapUtils.getFicheFieldsFilter(optionalQRequestMap);
        MotcleFieldsFilter motcleFieldsFilter = RequestMapUtils.getMotcleFieldsFilter(optionalQRequestMap);
        StartParameters startParameters = RequestMapUtils.getStartParameters(optionalQRequestMap, scrutariSession.getGlobalSearchOptions(), warningHandler);
        Lang langUi = searchResult.getFicheSearchSource().getSearchOptionsDef().getLangUi();
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        return new GeoJsonProducer(version, searchResult, scrutariSession.getScrutariDB(), scrutariSession.getGlobalSearchOptions(), ficheFieldsFilter, motcleFieldsFilter, startParameters, langUi);
    }

    private static FicheSearchResult buildFicheSearchResult(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler) throws ErrorMessageException {
        String qIdString = RequestMapUtils.getQIdString(requestMap);
        FicheSearchResult searchResult;
        if (qIdString != null) {
            if (qIdString.isEmpty()) {
                throw new ErrorMessageException(ParameterErrorMessageKeys.EMPTY_PARAMETER, Parameters.Q_ID);
            }
            try {
                QId qId = QId.parse(qIdString);
                searchResult = scrutariSession.getSearch(qId);
            } catch (ParseException pe) {
                throw new ErrorMessageException(ParameterErrorMessageKeys.WRONG_PARAMETER_VALUE, Parameters.Q_ID, qIdString);
            }
            if (searchResult == null) {
                throw new ErrorMessageException(ParameterErrorMessageKeys.UNKNOWN_PARAMETER_VALUE, Parameters.Q_ID, qIdString);
            }
        } else {
            SearchOperation searchOperation = RequestMapUtils.getSearchOperation(requestMap, scrutariSession.getScrutariDB().getAttributeKeyCoder(), warningHandler);
            SearchOptions searchOptions = RequestMapUtils.getSearchOptions(scrutariSession, requestMap, warningHandler, true);
            searchResult = scrutariSession.search(searchOperation, searchOptions);
            if ((searchResult.isNewSearch()) || (RequestMapUtils.isAllLog(requestMap))) {
                SctLogUtils.log(scrutariSession, searchResult, RequestMapUtils.getOrigin(requestMap));
            }
        }
        return searchResult;
    }

    private static AbstractJsonProducer getCategoryJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) {
        Lang langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        return new CategoryJsonProducer(version, scrutariSession.getGlobalSearchOptions(), langUi);
    }

    private static AbstractJsonProducer getBaseJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) {
        Lang langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        return new BaseJsonProducer(version, scrutariSession.getScrutariDB(), langUi);
    }

    private static AbstractJsonProducer getCorpusJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) {
        Lang langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        return new CorpusJsonProducer(version, scrutariSession.getScrutariDB(), langUi);
    }

    private static AbstractJsonProducer getThesaurusJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) {
        Lang langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        return new ThesaurusJsonProducer(version, scrutariSession.getScrutariDB(), langUi);
    }

    private static AbstractJsonProducer getLangJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) {
        Lang langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        Lang[] langArray = RequestMapUtils.getLangArray(requestMap);
        return new LangJsonProducer(scrutariSession, version, langUi, langArray);
    }

    private static AbstractJsonProducer getEngineInfoJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) {
        Lang langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        DataFilter dataFilter = RequestMapUtils.getDataFilter(requestMap);
        return new EngineInfoJsonProducer(version, scrutariSession, langUi, dataFilter);
    }

    private static AbstractJsonProducer getFicheJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) {
        Lang langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        CodeList ficheCodeList;
        String ficheListParam = requestMap.getParameter(Parameters.FICHE_LIST);
        if (ficheListParam != null) {
            ficheCodeList = RequestMapUtils.toCodeList(dataAccess, ficheListParam, warningHandler, "fiche:", ScrutariDataURI.FICHEURI_TYPE);
            if (ficheCodeList == null) {
                ficheCodeList = CodesUtils.EMPTY_CODELIST;
            }
        } else {
            SearchOptions searchOptions = RequestMapUtils.getSearchOptions(scrutariSession, requestMap, warningHandler, true);
            ficheCodeList = OptionsUtils.getFicheCodeList(dataAccess, searchOptions);
        }
        dataAccess.close();
        String randomString = requestMap.getParameter(Parameters.RANDOM);
        if (randomString != null) {
            warningHandler.setCurrentParameter(Parameters.RANDOM);
            try {
                int randomNumber = Integer.parseInt(randomString);
                if ((randomNumber == -1) || (randomNumber > 0)) {
                    ficheCodeList = CodesUtils.shuffle(ficheCodeList, randomNumber);
                } else {
                    warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, randomString);
                }
            } catch (NumberFormatException nfe) {
                warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, randomString);
            }
        }
        FicheFieldsFilter ficheFieldsFilter = RequestMapUtils.getFicheFieldsFilter(requestMap);
        return new FicheJsonProducer(version, scrutariSession.getScrutariDB(), ficheCodeList, ficheFieldsFilter, langUi);
    }

    private static AbstractJsonProducer getMotcleJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) {
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        CodeList motcleCodeList;
        String motcleListParam = requestMap.getParameter(Parameters.MOTCLE_LIST);
        if (motcleListParam != null) {
            motcleCodeList = RequestMapUtils.toCodeList(dataAccess, motcleListParam, warningHandler, "motcle:", ScrutariDataURI.MOTCLEURI_TYPE);
            if (motcleCodeList == null) {
                motcleCodeList = CodesUtils.EMPTY_CODELIST;
            }
        } else {
            SearchOptions searchOptions = RequestMapUtils.getSearchOptions(scrutariSession, requestMap, warningHandler, false);
            motcleCodeList = OptionsUtils.getMotcleCodeList(dataAccess, searchOptions);
        }
        MotcleFieldsFilter motcleFieldsFilter = RequestMapUtils.getMotcleFieldsFilter(requestMap);
        return new MotcleJsonProducer(version, scrutariDB, motcleCodeList, motcleFieldsFilter);
    }

    private static AbstractJsonProducer getFicheCountJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) {
        DataAccess dataAccess = scrutariSession.getScrutariDB().openDataAccess();
        CountFilter countFilter = RequestMapUtils.getCountFilter(dataAccess, requestMap, warningHandler);
        dataAccess.close();
        return new FicheCountJsonProducer(version, scrutariSession.getScrutariDB(), countFilter);
    }

    private static AbstractJsonProducer getEngineGroupJsonProducer(ScrutariSession scrutariSession, RequestMap requestMap, int version, WarningHandler warningHandler) throws ErrorMessageException {
        Lang langUi = RequestMapUtils.getLangUi(requestMap, warningHandler);
        if (langUi == null) {
            langUi = EngineUtils.getDefaultLangUi(scrutariSession);
        }
        return getEngineGroupJsonProducer(scrutariSession.getEngine().getEngineContext(), requestMap, version, langUi, warningHandler);
    }

    public static EngineGroupJsonProducer getEngineGroupJsonProducer(EngineContext engineContext, RequestMap requestMap, int version, Lang langUi, WarningHandler warningHandler) throws ErrorMessageException {
        String name = RequestMapUtils.getMandatoryParam(Parameters.NAME, requestMap);
        EngineGroup engineGroup = engineContext.getEngineGroup(name);
        if (engineGroup == null) {
            throw new ErrorMessageException(ParameterErrorMessageKeys.UNKNOWN_PARAMETER_VALUE, Parameters.NAME, name);
        }
        return new EngineGroupJsonProducer(version, engineContext, engineGroup, langUi, false);
    }

}
