/* UtilLib - Copyright (c) 2006-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import java.io.PrintWriter;
import java.io.Writer;
import java.util.Map;
import net.mapeadores.util.io.AppendableWriter;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.localisation.MessageLocalisationPack;


/**
 *
 * @author Vincent Calame
 */
public class HtmlPrinter {

    protected PrintWriter pw;
    private final MessageLocalisationPack messageLocalisationPack = new MessageLocalisationPack();
    private final MessageLocalisation messageLocalisation = messageLocalisationPack.getMessageLocalisationInterface();
    protected boolean breakLine = true;
    protected boolean withJavascript = false;

    public HtmlPrinter() {
    }

    public void addMessageLocalisation(MessageLocalisation messageLocalisation) {
        addMessageLocalisation(messageLocalisation, 1);
    }

    public void addMessageLocalisation(MessageLocalisation messageLocalisation, int priority) {
        messageLocalisationPack.addMessageLocalisation(messageLocalisation, priority);
    }

    public MessageLocalisation getMessageLocalisation() {
        return messageLocalisation;
    }

    public PrintWriter initPrinter(PrintWriter pw) {
        PrintWriter old = this.pw;
        this.pw = pw;
        return old;
    }

    public PrintWriter initPrinter(Appendable appendable) {
        PrintWriter old = this.pw;
        if (appendable instanceof PrintWriter) {
            this.pw = (PrintWriter) appendable;
        } else if (appendable instanceof Writer) {
            this.pw = new PrintWriter((Writer) appendable);
        } else {
            this.pw = new PrintWriter(new AppendableWriter(appendable));
        }
        return old;
    }

    public void setWithJavascript(boolean withJavascript) {
        this.withJavascript = withJavascript;
    }

    public boolean isWithJavascript() {
        return withJavascript;
    }

    public void flush() {
        pw.flush();
    }

    public PrintWriter getPrintWriter() {
        return pw;
    }

    public HtmlPrinter appendHtmlDocType() {
        pw.println("<!DOCTYPE html>");
        return this;
    }

    public HtmlPrinter appendFrameDocType() {
        pw.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Frameset//EN\" \"http://www.w3.org/TR/html4/frameset.dtd\">");
        return this;
    }

    public HtmlPrinter appendHtmlContentType() {
        openTag("meta");
        printAttribute("http-equiv", "Content-Type");
        printAttribute("content", "text/html; charset=UTF-8");
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter append(TrustedHtml trustedHtml) {
        pw.append(trustedHtml);
        return this;
    }

    public HtmlPrinter escape(char c) {
        return escape(c, false);
    }

    public HtmlPrinter escape(char carac, boolean preformatted) {
        switch (carac) {
            case '&':
                pw.print("&amp;");
                break;
            case '"':
                pw.print("&quot;");
                break;
            case '<':
                pw.print("&lt;");
                break;
            case '>':
                pw.print("&gt;");
                break;
            case '\'':
                pw.print("&#x27;");
                break;
            case '\\':
                pw.print("&#x5C;");
                break;
            case '`':
                pw.print("&#x60;");
                break;
            default:
                if ((!preformatted) && (carac < 32)) {
                    pw.print(' ');
                } else {
                    pw.print(carac);
                }
        }
        return this;
    }

    public HtmlPrinter append(int i) {
        pw.print(i);
        return this;
    }

    public HtmlPrinter append(long l) {
        pw.print(l);
        return this;
    }

    public HtmlPrinter space() {
        pw.print(' ');
        return this;
    }

    public HtmlPrinter newLine() {
        pw.print('\n');
        return this;
    }

    public HtmlPrinter nonBreakableSpace() {
        pw.print("&nbsp;");
        return this;
    }

    public HtmlPrinter doublespace() {
        pw.print("&nbsp; ");
        return this;
    }

    public HtmlPrinter escape(Object obj) {
        if (obj == null) {
            pw.print("null");
            return this;
        }
        return escape(obj.toString());
    }

    public HtmlPrinter escape(CharSequence s) {
        return escape(s, false);
    }

    public HtmlPrinter escape(CharSequence s, boolean preformatted) {
        if (s == null) {
            pw.print("null");
            return this;
        }
        int length = s.length();
        for (int i = 0; i < length; i++) {
            escape(s.charAt(i), preformatted);
        }
        return this;
    }


    public HtmlPrinter scriptLineEnd() {
        pw.println(";");
        return this;
    }

    public HtmlPrinter scriptLiteral(CharSequence s) {
        if (s == null) {
            pw.print("null");
            return this;
        }
        pw.print('"');
        int length = s.length();
        for (int i = 0; i < length; i++) {
            char carac = s.charAt(i);
            switch (carac) {
                case '\n':
                    pw.print("\\n");
                    break;
                case '\t':
                    pw.print("\\t");
                    break;
                case '\r':
                    pw.print("\\r");
                    break;
                case '"':
                    pw.print("\\\"");
                    break;
                case '\'':
                    pw.print("\\'");
                    break;
                case '\\':
                    pw.print("\\\\");
                    break;
                case '<':
                    pw.print("\\u003c");
                    break;
                case '>':
                    pw.print("\\u003e");
                    break;
                case '&':
                    pw.print("\\u0026");
                    break;
                default:
                    if (carac < 32) {
                        pw.print(' ');
                    } else {
                        pw.print(carac);
                    }
            }
        }
        pw.print('"');
        return this;
    }


    public String getLocalization(String messageKey) {
        String loc = messageLocalisation.toString(messageKey);
        if (loc == null) {
            loc = "?" + messageKey + "?";
        }
        return loc;
    }

    public String getLocalization(Message message) {
        String loc = messageLocalisation.toString(message);
        if (loc == null) {
            loc = "?" + message.getMessageKey() + "?";
        }
        return loc;
    }

    public HtmlPrinter localize(String messageKey) {
        escape(getLocalization(messageKey));
        return this;
    }

    public HtmlPrinter localize(Message message) {
        escape(getLocalization(message));
        return this;
    }

    public HtmlPrinter startA(HTMLAttributes htmlAttributes) {
        openTag("a");
        if (htmlAttributes != null) {
            printAttribute("href", htmlAttributes.href());
            printAttribute("target", htmlAttributes.target());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endA() {
        endTag("a");
        return this;
    }

    public HtmlPrinter startBody() {
        startBody(null);
        return this;
    }

    public HtmlPrinter startBody(HTMLAttributes htmlAttributes) {
        openTag("body");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endBody() {
        endTag("body");
        breakLine();
        return this;
    }

    public void appendBr() {
        startTag("br");
    }

    public void appendBr(int i) {
        for (int j = 0; j < i; j++) {
            appendBr();
        }
    }

    public HtmlPrinter startCell() {
        startTag("td");
        return this;
    }

    public HtmlPrinter startCell(HTMLAttributes htmlAttributes) {
        openTag("td");
        if (htmlAttributes != null) {
            int colSpan = htmlAttributes.colspan();
            if (colSpan > 1) {
                printAttribute("colspan", String.valueOf(colSpan));
            }
            int rowSpan = htmlAttributes.rowspan();
            if (rowSpan > 1) {
                printAttribute("rowspan", String.valueOf(rowSpan));
            }
            printAttribute("width", htmlAttributes.width());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endCell() {
        endTag("td");
        breakLine();
        return this;
    }

    public void appendEmptyCell() {
        startCell();
        endCell();
    }

    public HtmlPrinter startHeaderCell(HTMLAttributes htmlAttributes) {
        openTag("th");
        if (htmlAttributes != null) {
            int colSpan = htmlAttributes.colspan();
            if (colSpan > 1) {
                printAttribute("colspan", String.valueOf(colSpan));
            }
            int rowSpan = htmlAttributes.rowspan();
            if (rowSpan > 1) {
                printAttribute("rowspan", String.valueOf(rowSpan));
            }
            printAttribute("width", htmlAttributes.width());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endHeaderCell() {
        endTag("th");
        breakLine();
        return this;
    }

    public void appendCol(HTMLAttributes htmlAttributes) {
        openTag("col");
        if (htmlAttributes != null) {
            printAttribute("width", htmlAttributes.width());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public HtmlPrinter startCode() {
        startTag("code");
        return this;
    }

    public HtmlPrinter startCode(HTMLAttributes htmlAttributes) {
        openTag("code");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endCode() {
        endTag("code");
        return this;
    }

    public HtmlPrinter startDel(HTMLAttributes htmlAttributes) {
        openTag("del");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endDel() {
        endTag("del");
        return this;
    }

    public HtmlPrinter startEm() {
        startTag("em");
        return this;
    }

    public HtmlPrinter endEm() {
        endTag("em");
        return this;
    }

    public HtmlPrinter startDiv(HTMLAttributes htmlAttributes) {
        openTag("div");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endDiv() {
        endTag("div");
        breakLine();
        return this;
    }

    public HtmlPrinter startFieldset(HTMLAttributes htmlAttributes) {
        openTag("fieldset");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endFieldset() {
        endTag("fieldset");
        return this;
    }

    public HtmlPrinter startForm(HTMLAttributes htmlAttributes) {
        openTag("form");
        if (htmlAttributes != null) {
            printAttribute("action", htmlAttributes.action());
            printAttribute("method", htmlAttributes.method());
            printAttribute("target", htmlAttributes.target());
            printAttribute("enctype", htmlAttributes.enctype());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endForm() {
        endTag("form");
        return this;
    }

    public HtmlPrinter startH1(HTMLAttributes htmlAttributes) {
        openTag("h1");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endH1() {
        endTag("h1");
        breakLine();
        return this;
    }

    public HtmlPrinter startH2(HTMLAttributes htmlAttributes) {
        openTag("h2");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endH2() {
        endTag("h2");
        breakLine();
        return this;
    }

    public HtmlPrinter startH3(HTMLAttributes htmlAttributes) {
        openTag("h3");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endH3() {
        endTag("h3");
        breakLine();
        return this;
    }

    public HtmlPrinter startH4(HTMLAttributes htmlAttributes) {
        openTag("h4");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endH4() {
        endTag("h4");
        breakLine();
        return this;
    }

    public HtmlPrinter startH5(HTMLAttributes htmlAttributes) {
        openTag("h5");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endH5() {
        endTag("h5");
        breakLine();
        return this;
    }

    public HtmlPrinter startH6(HTMLAttributes htmlAttributes) {
        openTag("h6");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endH6() {
        endTag("h6");
        breakLine();
        return this;
    }

    public HtmlPrinter startHead() {
        startTag("head");
        breakLine();
        return this;
    }

    public HtmlPrinter endHead() {
        endTag("head");
        breakLine();
        return this;
    }

    public void appendHr(HTMLAttributes htmlAttributes) {
        openTag("hr");
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public HtmlPrinter startHtml(Lang lang) {
        openTag("html");
        if (lang != null) {
            printAttribute("lang", lang.toString());
            if (lang.isRTLScript()) {
                printAttribute("dir", "rtl");
            }

        }
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endHtml() {
        endTag("html");
        return this;
    }

    public void appendImg(HTMLAttributes htmlAttributes) {
        openTag("img");
        if (htmlAttributes != null) {
            printAttribute("src", htmlAttributes.src());
            printAttribute("alt", htmlAttributes.alt());
            printAttribute("width", htmlAttributes.width());
            printAttribute("height", htmlAttributes.height());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public HtmlPrinter startIframe(HTMLAttributes htmlAttributes) {
        openTag("iframe");
        if (htmlAttributes != null) {
            printAttribute("src", htmlAttributes.src());
            printAttribute("width", htmlAttributes.width());
            printAttribute("height", htmlAttributes.height());
            printAttribute("scrolling", htmlAttributes.scrolling());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endIframe() {
        endTag("iframe");
        breakLine();
        return this;
    }

    public HtmlPrinter startLegend(HTMLAttributes htmlAttributes) {
        openTag("legend");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endLegend() {
        endTag("legend");
        breakLine();
        return this;
    }

    public HtmlPrinter startLi() {
        startLi(null);
        return this;
    }

    public HtmlPrinter startLi(HTMLAttributes htmlAttributes) {
        openTag("li");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endLi() {
        endTag("li");
        breakLine();
        return this;
    }

    public void appendLink(HTMLAttributes htmlAttributes) {
        openTag("link");
        if (htmlAttributes != null) {
            printAttribute("href", htmlAttributes.href());
            printAttribute("rel", htmlAttributes.rel());
            printAttribute("type", htmlAttributes.type());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public void appendMeta(HTMLAttributes htmlAttributes) {
        openTag("meta");
        if (htmlAttributes != null) {
            printAttribute("http-equiv", htmlAttributes.httpEquiv());
            printAttribute("content", htmlAttributes.content());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
    }

    public HtmlPrinter startP() {
        startTag("p");
        return this;
    }

    public HtmlPrinter startP(HTMLAttributes htmlAttributes) {
        openTag("p");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter startP(String className) {
        openTag("p");
        printAttribute("class", className);
        closeTag();
        return this;
    }

    public HtmlPrinter endP() {
        endTag("p");
        breakLine();
        return this;
    }

    public HtmlPrinter startPre(HTMLAttributes htmlAttributes) {
        openTag("pre");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endPre() {
        endTag("pre");
        return this;
    }

    public HtmlPrinter startOption(HTMLAttributes htmlAttributes) {
        openTag("option");
        if (htmlAttributes != null) {
            printAttribute("value", htmlAttributes.value());
            printBooleanAttribute("selected", htmlAttributes.selected());
            printBooleanAttribute("disabled", htmlAttributes.disabled());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endOption() {
        endTag("option");
        breakLine();
        return this;
    }

    public HtmlPrinter startOptGroup(HTMLAttributes htmlAttributes) {
        openTag("optgroup");
        if (htmlAttributes != null) {
            printAttribute("label", htmlAttributes.label());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endOptGroup() {
        endTag("optgroup");
        breakLine();
        return this;
    }

    public HtmlPrinter startRow() {
        startTag("tr");
        breakLine();
        return this;
    }

    public HtmlPrinter startRow(HTMLAttributes htmlAttributes) {
        openTag("tr");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endRow() {
        endTag("tr");
        breakLine();
        return this;
    }

    public HtmlPrinter startScript(HTMLAttributes htmlAttributes) {
        openTag("script");
        if (htmlAttributes != null) {
            printAttribute("type", htmlAttributes.type());
            printAttribute("src", htmlAttributes.src());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endScript() {
        endTag("script");
        breakLine();
        return this;
    }

    public HtmlPrinter startNoscript() {
        startTag("noscript");
        return this;
    }

    public HtmlPrinter endNoscript() {
        endTag("noscript");
        return this;
    }

    public HtmlPrinter startSelect(HTMLAttributes htmlAttributes) {
        openTag("select");
        printHTMLAttributes(htmlAttributes);
        printBooleanAttribute("multiple", htmlAttributes.multiple());
        printAttribute("size", htmlAttributes.size());
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endSelect() {
        endTag("select");
        return this;
    }

    public HtmlPrinter startButton(HTMLAttributes htmlAttributes) {
        openTag("button");
        printHTMLAttributes(htmlAttributes);
        printBooleanAttribute("disabled", htmlAttributes.disabled());
        printAttribute("type", htmlAttributes.type());
        closeTag();
        return this;
    }

    public HtmlPrinter endButton() {
        endTag("button");
        return this;
    }

    public HtmlPrinter startSmall() {
        startTag("small");
        return this;
    }

    public HtmlPrinter endSmall() {
        endTag("small");
        return this;
    }

    public HtmlPrinter startSpan(HTMLAttributes htmlAttributes) {
        openTag("span");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endSpan() {
        endTag("span");
        return this;
    }

    public HtmlPrinter startStrong() {
        startTag("strong");
        return this;
    }

    public HtmlPrinter endStrong() {
        endTag("strong");
        return this;
    }

    public HtmlPrinter startTable() {
        startTable(null);
        return this;
    }

    public HtmlPrinter startTable(HTMLAttributes htmlAttributes) {
        openTag("table");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endTable() {
        endTag("table");
        breakLine();
        return this;
    }

    public HtmlPrinter startTHead(HTMLAttributes htmlAttributes) {
        openTag("thead");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endTHead() {
        endTag("thead");
        breakLine();
        return this;
    }

    public HtmlPrinter startTFoot(HTMLAttributes htmlAttributes) {
        openTag("tfoot");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endTFoot() {
        endTag("tfoot");
        breakLine();
        return this;
    }

    public HtmlPrinter startTBody(HTMLAttributes htmlAttributes) {
        openTag("tbody");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endTBody() {
        endTag("tbody");
        breakLine();
        return this;
    }

    public HtmlPrinter startTitle() {
        startTag("title");
        return this;
    }

    public HtmlPrinter endTitle() {
        endTag("title");
        breakLine();
        return this;
    }

    public HtmlPrinter startTextArea(HTMLAttributes htmlAttributes) {
        openTag("textarea");
        int cols = 0;
        int rows = 0;
        if (htmlAttributes != null) {
            cols = htmlAttributes.cols();
            rows = htmlAttributes.rows();
        }
        if (cols < 1) {
            cols = 50;
        }
        if (rows < 1) {
            rows = 3;
        }
        printAttribute("cols", String.valueOf(cols));
        printAttribute("rows", String.valueOf(rows));
        printBooleanAttribute("disabled", htmlAttributes.disabled());
        printBooleanAttribute("readonly", htmlAttributes.readonly());
        printBooleanAttribute("required", htmlAttributes.required());
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endTextArea() {
        endTag("textarea");
        return this;
    }

    public HtmlPrinter startUl() {
        startTag("ul");
        breakLine();
        return this;
    }

    public HtmlPrinter startUl(HTMLAttributes htmlAttributes) {
        openTag("ul");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        breakLine();
        return this;
    }

    public HtmlPrinter endUl() {
        endTag("ul");
        breakLine();
        return this;
    }

    public void appendInput(HTMLAttributes htmlAttributes) {
        appendInput(htmlAttributes, false);
    }

    public void appendInput(HTMLAttributes htmlAttributes, boolean preformatted) {
        openTag("input");
        if (htmlAttributes != null) {
            printAttribute("type", htmlAttributes.type());
            printPreformattedAttribute("value", htmlAttributes.value(), preformatted);
            printAttribute("size", htmlAttributes.size());
            printBooleanAttribute("checked", htmlAttributes.checked());
            printBooleanAttribute("disabled", htmlAttributes.disabled());
            printBooleanAttribute("readonly", htmlAttributes.readonly());
            printBooleanAttribute("required", htmlAttributes.required());
        }
        printHTMLAttributes(htmlAttributes);
        closeTag();
    }

    public void appendSmallestInput(String name, String value) {
        appendTextInput(name, value, 3);
    }

    public void appendSmallInput(String name, String value) {
        appendTextInput(name, value, 15);
    }

    public void appendMediumInput(String name, String value) {
        appendTextInput(name, value, 30);
    }

    public void appendLargeInput(String name, String value) {
        appendTextInput(name, value, 60);
    }

    public void appendHiddenInput(String name, String value) {
        openTag("input");
        printAttribute("type", HtmlConstants.HIDDEN_TYPE);
        printAttribute("name", name);
        printAttribute("value", value);
        closeTag();
    }

    private void appendTextInput(String name, String value, int size) {
        openTag("input");
        printAttribute("type", HtmlConstants.TEXT_TYPE);
        printAttribute("name", name);
        printAttribute("value", value);
        if (size > 0) {
            printAttribute("size", String.valueOf(size));
        }
        closeTag();
    }

    public HtmlPrinter startLabel(HTMLAttributes htmlAttributes) {
        openTag("label");
        printHTMLAttributes(htmlAttributes);
        closeTag();
        return this;
    }

    public HtmlPrinter endLabel() {
        endTag("label");
        return this;
    }

    public HtmlPrinter startComment() {
        pw.print("<!-- ");
        return this;
    }

    public HtmlPrinter endComment() {
        pw.print(" -->");
        return this;
    }

    public void breakLine() {
        if (breakLine) {
            pw.println("");
        }
    }

    private boolean printHTMLAttributes(HTMLAttributes htmlAttributes) {
        if (htmlAttributes == null) {
            return false;
        }
        printAttribute("class", htmlAttributes.classes());
        printAttribute("id", htmlAttributes.id());
        printAttribute("style", htmlAttributes.style());
        printAttribute("name", htmlAttributes.name());
        Map<String, String> attrMap = htmlAttributes.getAttrMap();
        if (attrMap != null) {
            for (Map.Entry<String, String> entry : attrMap.entrySet()) {
                pw.print(" ");
                escape(entry.getKey());
                pw.print("=\"");
                escape(entry.getValue());
                pw.print("\"");
            }
        }
        printAttribute("title", htmlAttributes.title());
        return true;
    }

    private void printAttribute(String name, String value) {
        if (value == null) {
            return;
        }
        pw.print(" ");
        pw.print(name);
        pw.print("=\"");
        escape(value);
        pw.print("\"");
    }

    private void printBooleanAttribute(String name, boolean value) {
        if (value) {
            printAttribute(name, name);
        }
    }

    private void startTag(String tag) {
        pw.print("<");
        pw.print(tag);
        pw.print(">");
    }

    private void openTag(String tag) {
        pw.print("<");
        pw.print(tag);
    }

    private void closeTag() {
        pw.print(">");
    }

    private void endTag(String tag) {
        pw.print("</");
        pw.print(tag);
        pw.print(">");
    }

    private void printPreformattedAttribute(String name, String value, boolean preformatted) {
        if (value == null) {
            return;
        }
        pw.print(" ");
        pw.print(name);
        pw.print("=\"");
        escape(value, preformatted);
        pw.print("\"");
    }

    public void startRowsFrameset(String rows) {
        openTag("frameset");
        printAttribute("rows", rows);
        closeTag();
        breakLine();
    }

    public void startColsFrameset(String cols) {
        openTag("frameset");
        printAttribute("cols", cols);
        closeTag();
        breakLine();
    }

    public void appendFrame(String name, String src) {
        openTag("frame");
        printAttribute("name", name);
        printAttribute("src", src);
        closeTag();
        breakLine();
    }

    public void endFrameset() {
        endTag("frameset");
        breakLine();
    }

}
