/* UtilLib - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.html;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;


/**
 *
 * @author Vincent Calame
 */
public class PageHtmlPrinter extends HtmlPrinter {

    private String htmlpage_title;
    private HTMLAttributes bodyHTMLAttributes;
    private CommandBoxParameters commandBoxParameters;
    private boolean onCommandBox = false;
    private String currentCommandKey = null;
    private final List<String> cssUrlList = new ArrayList<String>();
    private final List<String> jsUrlList = new ArrayList<String>();
    private String iconPng = null;

    public PageHtmlPrinter() {
    }

    public void setCommandBoxParameters(CommandBoxParameters commandBoxParameters) {
        this.commandBoxParameters = commandBoxParameters;
    }

    public CommandBoxParameters getCommandBoxParameters() {
        return commandBoxParameters;
    }

    public void setTitle(String title) {
        this.htmlpage_title = title;
    }

    public String getTitle() {
        return htmlpage_title;
    }

    public void setBodyHTMLAttributes(HTMLAttributes bodyHTMLAttributes) {
        this.bodyHTMLAttributes = bodyHTMLAttributes;
    }

    public void addCssUrl(String cssUrl) {
        cssUrlList.add(cssUrl);
    }

    public void addJsUrl(String jsUrl) {
        jsUrlList.add(jsUrl);
    }

    public void start(Lang lang) {
        appendHtmlDocType();
        startHtml(lang);
        startHead();
        startTitle();
        escape(htmlpage_title);
        endTitle();
        appendHtmlContentType();
        if (!jsUrlList.isEmpty()) {
            appendMeta(HA.httpEquiv("Content-Script-Type").content("text/javascript"));
        }
        for (String cssUrl : cssUrlList) {
            appendLink(HA.type("text/css").href(cssUrl).rel("stylesheet"));
        }
        if (iconPng != null) {
            appendLink(HA.type("image/png").href(iconPng).rel("icon"));
        }
        if (!jsUrlList.isEmpty()) {
            HTMLAttributes scriptAttributes = HA.type("text/javascript");
            int length = jsUrlList.size();
            for (int i = 0; i < length; i++) {
                scriptAttributes.src(jsUrlList.get(i));
                startScript(scriptAttributes);
                endScript();
            }
        }
        endHead();
        startBody(bodyHTMLAttributes);
    }

    public void startCommandBox(CommandBox commandBox) {
        if (onCommandBox) {
            throw new IllegalStateException("last CommandBox is not ended");
        }
        onCommandBox = true;
        HTMLAttributes formAttributes = HtmlPrinterUtils.newFormAttributes(commandBoxParameters);
        formAttributes.action(commandBox.getFormAction()).method(HtmlConstants.POST_METHOD);
        if (commandBox.isMultipart()) {
            formAttributes.enctype(HtmlConstants.MULTIPART_ENCTYPE);
        }
        String target = commandBox.getTarget();
        if (target != null) {
            formAttributes.target(target);
        }
        startForm(formAttributes);
        appendHiddenInput("cmd", commandBox.getCommandString());
        String page = commandBox.getPage();
        if (page != null) {
            appendHiddenInput("page", page);
        }
        String errorPage = commandBox.getErrorPage();
        if (errorPage != null) {
            appendHiddenInput("page-err", errorPage);
        }
        Map<String, String> hiddenAttributesMap = commandBox.getHiddenAttributesMap();
        if (hiddenAttributesMap != null) {
            for (Map.Entry<String, String> entry : hiddenAttributesMap.entrySet()) {
                appendHiddenInput(entry.getKey(), entry.getValue());
            }
        }
        HTMLAttributes fieldsetAttributes = HtmlPrinterUtils.newFieldsetAttributes(commandBoxParameters);
        String commandFamily = commandBox.getCommandFamily();
        if (commandFamily != null) {
            fieldsetAttributes.attr("data-command-family", commandFamily);
        } else {
            fieldsetAttributes.attr("data-command-family", null);
        }
        startFieldset(fieldsetAttributes);
        String commandKey = commandBox.getCommandKey();
        if (commandKey != null) {
            startLegend(HtmlPrinterUtils.newLegendAttributes(commandBoxParameters));
            localize(commandKey);
            endLegend();
            currentCommandKey = commandKey;
        }
    }

    public void endCommandBox(String submitMessageKey) {
        if (!onCommandBox) {
            throw new IllegalStateException("no CommandBox started yet");
        }
        if (submitMessageKey != null) {
            startDiv(HtmlPrinterUtils.newSubmitDivAttributes(commandBoxParameters));
            appendInput(HA.type(HtmlConstants.SUBMIT_TYPE).value(getLocalization(submitMessageKey)));
            endDiv();
        }
        if (currentCommandKey != null) {
            String keyString = currentCommandKey;
            if (keyString.startsWith("_ ")) {
                keyString = keyString.substring(2);
            }
            startDiv(HtmlPrinterUtils.newCommandKeyDivAttributes(commandBoxParameters));
            escape(keyString);
            endDiv();
        }
        endFieldset();
        endForm();
        currentCommandKey = null;
        onCommandBox = false;
    }

    public void end() {
        if (onCommandBox) {
            throw new IllegalStateException("last CommandBox is not ended");
        }
        endBody();
        endHtml();
    }

    public void setIconPng(String iconPng) {
        this.iconPng = iconPng;
    }

}
