/* UtilLib - Copyright (c) 2015-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.logging;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.ElementHandler;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class MessageDOMReader {

    private final MessageHandler messageHandler;

    public MessageDOMReader(MessageHandler messageHandler) {
        this.messageHandler = messageHandler;
    }

    public void readProcessMessageLog(MessageLogBuilder logBuilder, Element element, String xpath) {
        LogHandler logHandler = new LogHandler(logBuilder, xpath + "/" + element.getTagName());
        DOMUtils.readChildren(element, logHandler);
    }

    public CategorizedMessageList readList(Element element, String xpath) {
        CategorizedMessageListBuilder builder = new CategorizedMessageListBuilder();
        String uri = element.getAttribute("uri");
        if (uri.length() > 0) {
            xpath = xpath + "/" + element.getTagName() + "[@uri='" + uri + "']";
        } else {
            xpath = xpath + "/" + element.getTagName();
        }
        ListHandler listHandler = new ListHandler(builder, xpath);
        DOMUtils.readChildren(element, listHandler);
        return builder.toCategorizedMessageList();
    }

    private void addWarning(String messageKey, Object... messageValues) {
        messageHandler.addMessage("warning", LocalisationUtils.toMessage(messageKey, messageValues));
    }


    private class LogHandler implements ElementHandler {

        private final MessageLogBuilder logBuilder;
        private final String xpath;

        private LogHandler(MessageLogBuilder logBuilder, String xpath) {
            this.logBuilder = logBuilder;
            this.xpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            String xpath2 = xpath + "/" + tagName;
            if (tagName.equals("log")) {
                String uri = element.getAttribute("uri");
                if (uri.length() == 0) {
                    addWarning("_ error.empty.xml.attribute", xpath2, "uri");
                } else {
                    xpath2 = xpath2 + "[@uri='" + uri + "']";
                    logBuilder.setCurrentURI(uri);
                    MessageElementHandler messageElementHandler = new MessageElementHandler(logBuilder, xpath2);
                    DOMUtils.readChildren(element, messageElementHandler);
                }
            } else {
                addWarning("_ error.unknown.xml.tag", xpath2);
            }
        }

    }


    private class MessageElementHandler implements ElementHandler {

        private final MessageLogBuilder logBuilder;
        private final String xpath;


        private MessageElementHandler(MessageLogBuilder logBuilder, String xpath) {
            this.logBuilder = logBuilder;
            this.xpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            String xpath2 = xpath + "/" + tagName;
            if (tagName.equals("message")) {
                String key = element.getAttribute("key");
                if (key.length() == 0) {
                    addWarning("_ error.empty.xml.attribute", xpath2, "key");
                } else {
                    xpath2 = xpath2 + "[@key='" + key + "']";
                    String category = element.getAttribute("category");
                    List<Object> valueList = new ArrayList<Object>();
                    ValuesHandler valuesHandler = new ValuesHandler(valueList, xpath2);
                    DOMUtils.readChildren(element, valuesHandler);
                    logBuilder.addMessage(category, toMessage(key, valueList));
                }
            } else {
                addWarning("_ error.unknown.xml.tag", xpath2);
            }
        }

    }


    private class ListHandler implements ElementHandler {

        private final CategorizedMessageListBuilder builder;
        private final String xpath;


        private ListHandler(CategorizedMessageListBuilder builder, String xpath) {
            this.builder = builder;
            this.xpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            String xpath2 = xpath + "/" + tagName;
            if (tagName.equals("message")) {
                String key = element.getAttribute("key");
                if (key.length() == 0) {
                    addWarning("_ error.empty.xml.attribute", xpath2, "key");
                } else {
                    xpath2 = xpath2 + "[@key='" + key + "']";
                    String category = element.getAttribute("category");
                    List<Object> valueList = new ArrayList<Object>();
                    ValuesHandler valuesHandler = new ValuesHandler(valueList, xpath2);
                    DOMUtils.readChildren(element, valuesHandler);
                    builder.add(category, toMessage(key, valueList));
                }
            } else {
                addWarning("_ error.unknown.xml.tag", xpath2);
            }
        }

    }


    private class ValuesHandler implements ElementHandler {

        private final List<Object> list;
        private final String xpath;


        private ValuesHandler(List<Object> list, String xpath) {
            this.list = list;
            this.xpath = xpath;
        }

        @Override
        public void readElement(Element element) {
            String tagName = element.getTagName();
            String xpath2 = xpath + "/" + tagName;
            if (tagName.equals("value")) {
                String data = XMLUtils.getData(element);
                try {
                    Integer itg = Integer.parseInt(data);
                    list.add(itg);
                } catch (NumberFormatException nfe) {
                    list.add(data);
                }
            } else {
                addWarning("_ error.unknown.xml.tag", xpath2);
            }
        }

    }

    private static Message toMessage(String key, List<Object> valueList) {
        int size = valueList.size();
        if (size == 0) {
            return LocalisationUtils.toMessage(key);
        } else {
            return LocalisationUtils.toMessage(key, valueList.toArray(new Object[size]));
        }
    }


}
